(* ::Package:: *)

BeginPackage["qm`qmAlgebra`",
{"qm`qmBasic`","qm`qmTypesetting`",
"qm`qmSlots`","qm`qmSlotsPrimitives`","qm`qmSlotsExpressions`",
"qm`qmOperations`",
"Global`"}];


(* 
   Copyright 2017
   Jose Luis Gomez-Munoz and Francisco Delgado
   ITESM-CEM, 
   Departamento de F\[IAcute]sica y Matem\[AAcute]ticas (Ciencias B\[AAcute]sicas)
   Carretera Lago de Guadalupe Km. 3.5, 
   Atizap\[AAcute]n de Zaragoza, Estado de M\[EAcute]xico, 
   C.P. 52926
   Mexico
   jose.luis.gomez@itesm.mx
   http://homepage.cem.itesm.mx/jose.luis.gomez/
*)


Unprotect[qi`$algebraNames];
ClearAll[qi`$algebraNames];
qi`$algebraNames=HoldComplete[
			(* end-user qmcommands *)
			qm`Dirac`qmEvaluateSymmetrized,
			qm`Dirac`qmExpandCommutators,
			qm`Dirac`qmEvaluateCommutators,
			qm`Dirac`qmArrangeProducts,
			qm`Dirac`qmReverseProducts,
			qm`Dirac`qmArrangePowers,
			qm`Dirac`qmUnfoldPowers,
			qm`Dirac`qmCollectFromLeft,
			qm`Dirac`qmCollectFromRight,
			qm`Dirac`qmExpandProducts,
			qm`Dirac`qmSimplify,
			qm`Dirac`qmFullSimplify,
			(* qe`commands *)
			qe`evaluatesymmetrized,
			qe`expandcommutators,
			qe`evaluatecommutators,
			qe`arrangeProducts,
			qe`reverseProducts,
			qe`arrangePowers,
			qe`unfoldPowers,
			qe`collectFromLeft,
			qe`collectFromRight,
			qe`expandProducts,
			qe`simplify,
			qe`fullsimplify,
			qe`fullform,
			(* programmer commands *)
			qi`arrangeOutsideFlag,
			qi`qvalue,
			(* programmer lists *)
			qi`$notInAlgebraRules,
			qi`$symmevalRules,
			qi`$commexpandRules,
			qi`$commevalRules,
			qi`$arrangeProductsRules,
			qi`$reverseProductsRules,
			qi`$arrangeRules,
			qi`$unfoldRules,
			qi`$leftcollectRules,
			qi`$rightcollectRules,
			qi`$expandProductsRules,
			qi`$simplifyFunctions
];
Apply[Unprotect,qi`$algebraNames];
Apply[ClearAll,qi`$algebraNames];
Scan[Function[#::usage=ToString[#]<>" is a symbol of the Quantum Mechanics packages"],
	qi`$algebraNames];
(* Next command does NOT protect the names (symbols) inside the list qi`$algebraNames;
   it DOES protect the name of the list itself *)
SetAttributes[qi`$algebraNames,{Protected,ReadProtected}];
qi`$algebraNames::usage=
	"qi`$algebraNames is an internal symbol of the Quantum Mechanics packages";


qi`setClearAllSymbols[
	qm`Dirac`qmEvaluateSymmetrized,
	qm`Dirac`qmExpandCommutators,
	qm`Dirac`qmEvaluateCommutators,
	qm`Dirac`qmArrangeProducts,
	qm`Dirac`qmReverseProducts,
	qm`Dirac`qmArrangePowers,
	qm`Dirac`qmUnfoldPowers,
	qm`Dirac`qmCollectFromLeft,
	qm`Dirac`qmCollectFromRight,
	qm`Dirac`qmExpandProducts,
	qm`Dirac`qmSimplify,
	qm`Dirac`qmFullSimplify
];


Begin["`Private`"];


qi`setPreRules[
	HoldPattern[qm`Dirac`qmEvaluateSymmetrized]->qe`evaluatesymmetrized,
	HoldPattern[qm`Dirac`qmExpandCommutators]->qe`expandcommutators,
	HoldPattern[qm`Dirac`qmEvaluateCommutators]->qe`evaluatecommutators,
	HoldPattern[qm`Dirac`qmArrangeProducts]->qe`arrangeProducts,
	HoldPattern[qm`Dirac`qmReverseProducts]->qe`reverseProducts,
	HoldPattern[qm`Dirac`qmArrangePowers]->qe`arrangePowers,
	HoldPattern[qm`Dirac`qmUnfoldPowers]->qe`unfoldPowers,
	HoldPattern[qm`Dirac`qmCollectFromLeft]->qe`collectFromLeft,
	HoldPattern[qm`Dirac`qmCollectFromRight]->qe`collectFromRight,
	HoldPattern[qm`Dirac`qmExpandProducts]->qe`expandProducts,
	HoldPattern[qm`Dirac`qmSimplify]->qe`simplify, 
	HoldPattern[qm`Dirac`qmFullSimplify]->qe`fullsimplify,
	HoldPattern[qm`Dirac`qmFullForm]->qe`fullform
];

qi`setPostRules[
	HoldPattern[qe`evaluatesymmetrized]->qm`Dirac`qmEvaluateSymmetrized,
	HoldPattern[qe`expandcommutators]->qm`Dirac`qmExpandCommutators,
	HoldPattern[qe`evaluatecommutators]->qm`Dirac`qmEvaluateCommutators,
	HoldPattern[qe`arrangeProducts]->qm`Dirac`qmArrangeProducts,
	HoldPattern[qe`reverseProducts]->qm`Dirac`qmReverseProducts,
	HoldPattern[qe`arrangePowers]->qm`Dirac`qmArrangePowers,
	HoldPattern[qe`unfoldPowers]->qm`Dirac`qmUnfoldPowers,
	HoldPattern[qe`collectFromLeft]->qm`Dirac`qmCollectFromLeft,
	HoldPattern[qe`collectFromRight]->qm`Dirac`qmCollectFromRight,
	HoldPattern[qe`expandProducts]->qm`Dirac`qmExpandProducts,
	HoldPattern[qe`simplify]->qm`Dirac`qmSimplify,
	HoldPattern[qe`fullsimplify]->qm`Dirac`qmFullSimplify,
	HoldPattern[qe`fullform]->qm`Dirac`qmFullForm
];


(* When qmFullForm is inside qm[], it becomes qe`fullform, 
with a string as an output. *)

qi`setUndefined[qe`fullform];

HoldPattern[qe`fullform[arg_]]:=
Block[{qi`qmarg},
		qi`logPrint["qeform"," enter qeform"];
		qi`qmarg=HoldComplete[arg];
		qi`logPrint["qeform"," qeform1 ",qi`qmarg];
		qi`qmarg=qi`qmarg//.qi`$preRules;
		qi`logPrint["qeform"," qeform2 ",qi`qmarg];
		ToString[ReleaseHold[qi`qmarg]]
]


(* Lists of algebraic transformations *)


qi`$notInAlgebraRules={
	HoldPattern[qe`plus[args__]]:>
		With[{sargs=Sort[{args},qi`normalOrderQ]},qe`plus@@sargs/;sargs=!={args}]
};


qi`$leftcollectRules={
HoldPattern[qe`plus[plusbefore$___,common_?qe`nonscalarQ,plusbetween$___,qe`power[common_,exponsecond_?IntegerQ],plusafter$___]]:>qe`plus[plusbefore$,plusbetween$,qe`times[common,qe`plus[1,qe`power[common,-1+exponsecond]]],plusafter$],
HoldPattern[qe`plus[plusbefore$___,common_?qe`nonscalarQ,plusbetween$___,qe`times[scasecond:_?qe`scalarQ:1,common_,timessecond___],plusafter$___]]:>qe`plus[plusbefore$,plusbetween$,qe`times[common,qe`plus[1,qe`times[scasecond,timessecond]]],plusafter$],
HoldPattern[qe`plus[plusbefore$___,common_?qe`nonscalarQ,plusbetween$___,qe`times[scasecond:_?qe`scalarQ:1,qe`power[common_,exponsecond_?IntegerQ],timessecond___],plusafter$___]]:>qe`plus[plusbefore$,plusbetween$,qe`times[common,qe`plus[1,qe`times[scasecond,qe`power[common,-1+exponsecond],timessecond]]],plusafter$],
HoldPattern[qe`plus[plusbefore$___,qe`power[common_?qe`nonscalarQ,exponfirst_?IntegerQ],plusbetween$___,common_,plusafter$___]]:>qe`plus[plusbefore$,plusbetween$,qe`times[common,qe`plus[qe`power[common,-1+exponfirst],1]],plusafter$],
HoldPattern[qe`plus[plusbefore$___,qe`power[common_?qe`nonscalarQ,exponfirst_?IntegerQ],plusbetween$___,qe`power[common_,exponsecond_?IntegerQ],plusafter$___]]:>qe`plus[plusbefore$,plusbetween$,qe`times[common,qe`plus[qe`power[common,-1+exponfirst],qe`power[common,-1+exponsecond]]],plusafter$],
HoldPattern[qe`plus[plusbefore$___,qe`power[common_?qe`nonscalarQ,exponfirst_?IntegerQ],plusbetween$___,qe`times[scasecond:_?qe`scalarQ:1,common_,timessecond___],plusafter$___]]:>qe`plus[plusbefore$,plusbetween$,qe`times[common,qe`plus[qe`power[common,-1+exponfirst],qe`times[scasecond,timessecond]]],plusafter$],
HoldPattern[qe`plus[plusbefore$___,qe`power[common_?qe`nonscalarQ,exponfirst_?IntegerQ],plusbetween$___,qe`times[scasecond:_?qe`scalarQ:1,qe`power[common_,exponsecond_?IntegerQ],timessecond___],plusafter$___]]:>qe`plus[plusbefore$,plusbetween$,qe`times[common,qe`plus[qe`power[common,-1+exponfirst],qe`times[scasecond,qe`power[common,-1+exponsecond],timessecond]]],plusafter$],
HoldPattern[qe`plus[plusbefore$___,qe`times[scafirst:_?qe`scalarQ:1,common_?qe`nonscalarQ,timesfirst___],plusbetween$___,common_,plusafter$___]]:>qe`plus[plusbefore$,plusbetween$,qe`times[common,qe`plus[qe`times[scafirst,timesfirst],1]],plusafter$],
HoldPattern[qe`plus[plusbefore$___,qe`times[scafirst:_?qe`scalarQ:1,common_?qe`nonscalarQ,timesfirst___],plusbetween$___,qe`power[common_,exponsecond_?IntegerQ],plusafter$___]]:>qe`plus[plusbefore$,plusbetween$,qe`times[common,qe`plus[qe`times[scafirst,timesfirst],qe`power[common,-1+exponsecond]]],plusafter$],
HoldPattern[qe`plus[plusbefore$___,qe`times[scafirst:_?qe`scalarQ:1,common_?qe`nonscalarQ,timesfirst___],plusbetween$___,qe`times[scasecond:_?qe`scalarQ:1,common_,timessecond___],plusafter$___]]:>qe`plus[plusbefore$,plusbetween$,qe`times[common,qe`plus[qe`times[scafirst,timesfirst],qe`times[scasecond,timessecond]]],plusafter$],
HoldPattern[qe`plus[plusbefore$___,qe`times[scafirst:_?qe`scalarQ:1,common_?qe`nonscalarQ,timesfirst___],plusbetween$___,qe`times[scasecond:_?qe`scalarQ:1,qe`power[common_,exponsecond_?IntegerQ],timessecond___],plusafter$___]]:>qe`plus[plusbefore$,plusbetween$,qe`times[common,qe`plus[qe`times[scafirst,timesfirst],qe`times[scasecond,qe`power[common,-1+exponsecond],timessecond]]],plusafter$],
HoldPattern[qe`plus[plusbefore$___,qe`times[scafirst:_?qe`scalarQ:1,qe`power[common_?qe`nonscalarQ,exponfirst_?IntegerQ],timesfirst___],plusbetween$___,common_,plusafter$___]]:>qe`plus[plusbefore$,plusbetween$,qe`times[common,qe`plus[qe`times[scafirst,qe`power[common,-1+exponfirst],timesfirst],1]],plusafter$],
HoldPattern[qe`plus[plusbefore$___,qe`times[scafirst:_?qe`scalarQ:1,qe`power[common_?qe`nonscalarQ,exponfirst_?IntegerQ],timesfirst___],plusbetween$___,qe`power[common_,exponsecond_?IntegerQ],plusafter$___]]:>qe`plus[plusbefore$,plusbetween$,qe`times[common,qe`plus[qe`times[scafirst,qe`power[common,-1+exponfirst],timesfirst],qe`power[common,-1+exponsecond]]],plusafter$],
HoldPattern[qe`plus[plusbefore$___,qe`times[scafirst:_?qe`scalarQ:1,qe`power[common_?qe`nonscalarQ,exponfirst_?IntegerQ],timesfirst___],plusbetween$___,qe`times[scasecond:_?qe`scalarQ:1,common_,timessecond___],plusafter$___]]:>qe`plus[plusbefore$,plusbetween$,qe`times[common,qe`plus[qe`times[scafirst,qe`power[common,-1+exponfirst],timesfirst],qe`times[scasecond,timessecond]]],plusafter$],
HoldPattern[qe`plus[plusbefore$___,qe`times[scafirst:_?qe`scalarQ:1,qe`power[common_?qe`nonscalarQ,exponfirst_?IntegerQ],timesfirst___],plusbetween$___,qe`times[scasecond:_?qe`scalarQ:1,qe`power[common_,exponsecond_?IntegerQ],timessecond___],plusafter$___]]:>qe`plus[plusbefore$,plusbetween$,qe`times[common,qe`plus[qe`times[scafirst,qe`power[common,-1+exponfirst],timesfirst],qe`times[scasecond,qe`power[common,-1+exponsecond],timessecond]]],plusafter$]}


qi`$rightcollectRules={
HoldPattern[qe`plus[plusbefore$___,common_?qe`nonscalarQ,plusbetween$___,qe`power[common_,exponsecond_?IntegerQ],plusafter$___]]:>qe`plus[plusbefore$,plusbetween$,qe`times[qe`plus[1,qe`power[common,-1+exponsecond]],common],plusafter$],
HoldPattern[qe`plus[plusbefore$___,common_?qe`nonscalarQ,plusbetween$___,qe`times[timessecond___,common_],plusafter$___]]:>qe`plus[plusbefore$,plusbetween$,qe`times[qe`plus[1,qe`times[timessecond]],common],plusafter$],
HoldPattern[qe`plus[plusbefore$___,common_?qe`nonscalarQ,plusbetween$___,qe`times[timessecond___,qe`power[common_,exponsecond_?IntegerQ]],plusafter$___]]:>qe`plus[plusbefore$,plusbetween$,qe`times[qe`plus[1,qe`times[timessecond,qe`power[common,-1+exponsecond]]],common],plusafter$],
HoldPattern[qe`plus[plusbefore$___,qe`power[common_?qe`nonscalarQ,exponfirst_?IntegerQ],plusbetween$___,common_,plusafter$___]]:>qe`plus[plusbefore$,plusbetween$,qe`times[qe`plus[qe`power[common,-1+exponfirst],1],common],plusafter$],
HoldPattern[qe`plus[plusbefore$___,qe`power[common_?qe`nonscalarQ,exponfirst_?IntegerQ],plusbetween$___,qe`power[common_,exponsecond_?IntegerQ],plusafter$___]]:>qe`plus[plusbefore$,plusbetween$,qe`times[qe`plus[qe`power[common,-1+exponfirst],qe`power[common,-1+exponsecond]],common],plusafter$],
HoldPattern[qe`plus[plusbefore$___,qe`power[common_?qe`nonscalarQ,exponfirst_?IntegerQ],plusbetween$___,qe`times[timessecond___,common_],plusafter$___]]:>qe`plus[plusbefore$,plusbetween$,qe`times[qe`plus[qe`power[common,-1+exponfirst],qe`times[timessecond]],common],plusafter$],
HoldPattern[qe`plus[plusbefore$___,qe`power[common_?qe`nonscalarQ,exponfirst_?IntegerQ],plusbetween$___,qe`times[timessecond___,qe`power[common_,exponsecond_?IntegerQ]],plusafter$___]]:>qe`plus[plusbefore$,plusbetween$,qe`times[qe`plus[qe`power[common,-1+exponfirst],qe`times[timessecond,qe`power[common,-1+exponsecond]]],common],plusafter$],
HoldPattern[qe`plus[plusbefore$___,qe`times[timesfirst___,common_?qe`nonscalarQ],plusbetween$___,common_,plusafter$___]]:>qe`plus[plusbefore$,plusbetween$,qe`times[qe`plus[qe`times[timesfirst],1],common],plusafter$],
HoldPattern[qe`plus[plusbefore$___,qe`times[timesfirst___,common_?qe`nonscalarQ],plusbetween$___,qe`power[common_,exponsecond_?IntegerQ],plusafter$___]]:>qe`plus[plusbefore$,plusbetween$,qe`times[qe`plus[qe`times[timesfirst],qe`power[common,-1+exponsecond]],common],plusafter$],
HoldPattern[qe`plus[plusbefore$___,qe`times[timesfirst___,common_?qe`nonscalarQ],plusbetween$___,qe`times[timessecond___,common_],plusafter$___]]:>qe`plus[plusbefore$,plusbetween$,qe`times[qe`plus[qe`times[timesfirst],qe`times[timessecond]],common],plusafter$],
HoldPattern[qe`plus[plusbefore$___,qe`times[timesfirst___,common_?qe`nonscalarQ],plusbetween$___,qe`times[timessecond___,qe`power[common_,exponsecond_?IntegerQ]],plusafter$___]]:>qe`plus[plusbefore$,plusbetween$,qe`times[qe`plus[qe`times[timesfirst],qe`times[timessecond,qe`power[common,-1+exponsecond]]],common],plusafter$],
HoldPattern[qe`plus[plusbefore$___,qe`times[timesfirst___,qe`power[common_?qe`nonscalarQ,exponfirst_?IntegerQ]],plusbetween$___,common_,plusafter$___]]:>qe`plus[plusbefore$,plusbetween$,qe`times[qe`plus[qe`times[timesfirst,qe`power[common,-1+exponfirst]],1],common],plusafter$],
HoldPattern[qe`plus[plusbefore$___,qe`times[timesfirst___,qe`power[common_?qe`nonscalarQ,exponfirst_?IntegerQ]],plusbetween$___,qe`power[common_,exponsecond_?IntegerQ],plusafter$___]]:>qe`plus[plusbefore$,plusbetween$,qe`times[qe`plus[qe`times[timesfirst,qe`power[common,-1+exponfirst]],qe`power[common,-1+exponsecond]],common],plusafter$],
HoldPattern[qe`plus[plusbefore$___,qe`times[timesfirst___,qe`power[common_?qe`nonscalarQ,exponfirst_?IntegerQ]],plusbetween$___,qe`times[timessecond___,common_],plusafter$___]]:>qe`plus[plusbefore$,plusbetween$,qe`times[qe`plus[qe`times[timesfirst,qe`power[common,-1+exponfirst]],qe`times[timessecond]],common],plusafter$],
HoldPattern[qe`plus[plusbefore$___,qe`times[timesfirst___,qe`power[common_?qe`nonscalarQ,exponfirst_?IntegerQ]],plusbetween$___,qe`times[timessecond___,qe`power[common_,exponsecond_?IntegerQ]],plusafter$___]]:>qe`plus[plusbefore$,plusbetween$,qe`times[qe`plus[qe`times[timesfirst,qe`power[common,-1+exponfirst]],qe`times[timessecond,qe`power[common,-1+exponsecond]]],common],plusafter$]}


ClearAll[qi`qvalue];
qi`$arrangeProductsRules={
	HoldPattern[
		qe`times[first___,b_?qe`nonscalarQ,a_?qe`nonscalarQ,second___]/;
					And[FreeQ[{a,b},qe`ket],FreeQ[{a,b},qe`bra],
						Not[qi`normalOrderQ[b,a]]]]:>
		qe`plus[
			qe`times[Power[qi`qvalue,-1],first,a,b,second],
			qe`times[Times[-1,Power[qi`qvalue,-1]],
				first,qe`qmutator[a,b,qi`qvalue],second]]
};
qi`$reverseProductsRules={
	HoldPattern[
		qe`times[first___,b_?qe`nonscalarQ,a_?qe`nonscalarQ,second___]/;
					And[FreeQ[{a,b},qe`ket],FreeQ[{a,b},qe`bra],
						Block[{qi`reverseOrderingFlag=True},
							Not[qi`normalOrderQ[b,a]]]]]:>
		Block[{qi`reverseOrderingFlag=True},
			qe`plus[
				qe`times[Power[qi`qvalue,-1],first,a,b,second],
				qe`times[Times[-1,Power[qi`qvalue,-1]],
					first,qe`qmutator[a,b,qi`qvalue],second]] ]
};
qi`qvalue=1;


qi`$commexpandRules={
HoldPattern[qe`qmutator[a_qe`qmutator,b_,q_]]:>qe`plus[qe`times[a,b],qe`times[-q,b,a]],
HoldPattern[qe`qmutator[a_,b_qe`qmutator,q_]]:>qe`plus[qe`times[a,b],qe`times[-q,b,a]],

HoldPattern[qe`qmutator[a_?qe`nonscalarQ,qe`power[b_?qe`nonscalarQ,m_?IntegerQ],q_]/;
			And[(m>1),FreeQ[{a,b},qe`ket],FreeQ[{a,b},qe`bra]]]:>
	qe`plus[qe`times[b,qe`qmutator[a,qe`power[b,m-1],q]],
			qe`times[qe`qmutator[a,b,q],qe`power[b,m-1]],
			qe`times[q-1,b,a,qe`power[b,m-1]]],

HoldPattern[qe`qmutator[qe`power[a_?qe`nonscalarQ,n_?IntegerQ],b_?qe`nonscalarQ,q_]/;
			And[(n>1),FreeQ[{a,b},qe`ket],FreeQ[{a,b},qe`bra]]]:>
	qe`plus[qe`times[a,qe`qmutator[qe`power[a,n-1],b,q]],
			qe`times[qe`qmutator[a,b,q],qe`power[a,n-1]],
			qe`times[q-1,a,b,qe`power[a,n-1]]],
			
HoldPattern[qe`qmutator[qe`times[a_?qe`nonscalarQ,cc__],b_?qe`nonscalarQ,q_]/;
				And[FreeQ[{a,cc,b},qe`ket],FreeQ[{a,cc,b},qe`bra]]]:>
	qe`plus[qe`times[qe`qmutator[a,b,q],qe`times[cc]],
			qe`times[a,qe`qmutator[qe`times[cc],b,q]],
			qe`times[q-1,a,b,qe`times[cc]]],
			
HoldPattern[qe`qmutator[a_?qe`nonscalarQ,qe`times[b_?qe`nonscalarQ,dd__],q_]/;
				And[FreeQ[{a,b,dd},qe`ket],FreeQ[{a,b,dd},qe`bra]]]:>
	qe`plus[qe`times[qe`qmutator[a,b,q],qe`times[dd]],
			qe`times[b,qe`qmutator[a,qe`times[dd],q]],
			qe`times[q-1,b,a,qe`times[dd]]],

HoldPattern[qe`qmutator[qe`plus[a_,cc__],b_,q_]]:>
	qe`plus[qe`qmutator[a,b,q],qe`qmutator[qe`plus[cc],b,q]],
	
HoldPattern[qe`qmutator[a_,qe`plus[b_,dd__],q_]]:>
	qe`plus[qe`qmutator[a,b,q],qe`qmutator[a,qe`plus[dd],q]]
};


qi`$symmevalRules={
	HoldPattern[qe`symmetric[qe`times[args__]]]:>
		qe`plus[qe`times[1/2,args],qe`times@@Reverse[{args,1/2}]]
};


qi`$commevalRules={
	HoldPattern[qe`qmutator[a_,b_,q_]]:>qe`plus[qe`times[a,b],qe`times[-q,b,a]]
};


qi`arrangeOutsideFlag=True;
qi`$arrangeRules=
	{HoldPattern[
		qe`times[argsleft___,qe`power[qe`times[argspow___],n1_Integer],argsright___]/;
			qi`arrangeOutsideFlag]:>
		Block[{qi`arrangeOutsideFlag=False},
			Apply[qe`times,{argsleft,
						Apply[Sequence,Apply[Sequence,Table[{argspow}, n1],{1}]],
						argsright}]],
	HoldPattern[qe`times[
		args1___, 
		xy__,qe`power[qe`times[xy__],n_.], 
		args2___
		]]:> qe`times[args1, qe`power[qe`times[xy],(1+n)], args2] /; 
			qe`scalarQ[n],
	HoldPattern[qe`times[
		args1___, 
		qe`power[qe`times[xy__],m_.],xy__, 
		args2___
		]]:> qe`times[args1, qe`power[qe`times[xy],(m+1)], args2] /; 
			qe`scalarQ[m],
	HoldPattern[qe`times[
		args1___, 
		xy__,xy__, 
		args2___
		]]:> qe`times[args1, qe`power[qe`times[xy],2], args2]
	};


qi`$unfoldRules={
	HoldPattern[
		qe`power[qe`times[argspow___],n1_Integer]/;
			qi`arrangeOutsideFlag]:>
		Block[{qi`arrangeOutsideFlag=False},
			Apply[qe`times,{
						Apply[Sequence,Apply[Sequence,Table[{argspow}, n1],{1}]]}]],
						
	HoldPattern[
		qe`times[argsleft___,qe`power[qe`times[argspow___],n1_Integer],argsright___]/;
			qi`arrangeOutsideFlag]:>
		Block[{qi`arrangeOutsideFlag=False},
			Apply[qe`times,{argsleft,
						Apply[Sequence,Apply[Sequence,Table[{argspow}, n1],{1}]],
						argsright}]]
};


qi`$expandProductsRules=
	{HoldPattern[qe`times[before___,
						qe`plus[plusargs__],
						right_,
						after___]]:>
				qe`times[before,
						Apply[qe`plus,Map[qe`times[#1,right]&,{plusargs}]],
						after],

	HoldPattern[qe`times[before___,
						left_,
						qe`plus[plusargs__],
						after___]]:>
				qe`times[before,
						Apply[qe`plus,Map[qe`times[left,#1]&,{plusargs}]],
						after],

	HoldPattern[qe`power[qe`plus[first_,rest__],
						exponent_?(IntegerQ[#]&&(#>1) &)]]:>
			qe`plus @@ (qe`times @@@ Tuples[{first,qe`plus[rest]},exponent])};


qi`$simplifyFunctions=
	{qe`expandcommutators,
	qe`evaluatecommutators,
	qe`arrangeProducts,
	qe`reverseProducts,
	qe`arrangePowers,
	qe`unfoldPowers,
	qe`collectFromLeft,
	qe`collectFromRight,
	qe`expandProducts,
	ExpandAll};


(*These lists are protected*)
Protect[
	qi`$notInAlgebraRules,
	qi`$symmevalRules,
	qi`$commexpandRules,
	qi`$commevalRules,
	qi`$arrangeProductsRules,
	qi`$reverseProductsRules,
	qi`$arrangeRules,
	qi`$unfoldRules,
	qi`$leftcollectRules,
	qi`$rightcollectRules,
	qi`$expandProductsRules,
	qi`$simplifyFunctions
];
(*Data lists are read protected. Information lists are not*)
SetAttributes[{
		qi`$notInAlgebraRules,
		qi`$symmevalRules,
		qi`$commexpandRules,
		qi`$commevalRules,
		qi`$arrangeProductsRules,
		qi`$reverseProductsRules,
		qi`$arrangeRules,
		qi`$unfoldRules,
		qi`$leftcollectRules,
		qi`$rightcollectRules,
		qi`$expandProductsRules,
		qi`$simplifyFunctions
	},
	ReadProtected
];


Scan[Function[lista,
With[{comandoqe=lista[[1]],comandoqm=lista[[2]]},Module[{unprotected},
	unprotected=Unprotect[comandoqe, comandoqm];
(* remember the rule for sym implies the rules for
 sym[], sym[args] and even sym[args1][args2], 
because of a previous general rule of qi`slotlabels *)
	comandoqm /: qi`slotlabels[HoldPattern[comandoqm]]:=
		Missing["Undefined"];
	comandoqe /: qi`slotlabels[HoldPattern[comandoqe]]:=
		Missing["Undefined"];
	comandoqe /: qi`slotlabels[HoldPattern[comandoqe[arg1_,rest___]]]:=
	Module[{labels},
		labels=qi`slotlabels[arg1];
		qi`verifySlotLabels[labels]
	];
	Protect[Evaluate[unprotected]] ] ] ],
{
{qe`evaluatesymmetrized, qm`Dirac`qmEvaluateSymmetrized},
{qe`expandcommutators, qm`Dirac`qmExpandCommutators},
{qe`evaluatecommutators, qm`Dirac`qmEvaluateCommutators},
{qe`arrangeProducts, qm`Dirac`qmArrangeProducts},
{qe`reverseProducts, qm`Dirac`qmReverseProducts},
{qe`arrangePowers, qm`Dirac`qmArrangePowers},
{qe`unfoldPowers, qm`Dirac`qmUnfoldPowers},
{qe`collectFromLeft, qm`Dirac`qmCollectFromLeft},
{qe`collectFromRight, qm`Dirac`qmCollectFromRight},
{qe`expandProducts, qm`Dirac`qmExpandProducts},
{qe`simplify, qm`Dirac`qmSimplify},
{qe`fullsimplify, qm`Dirac`qmFullSimplify}} ];


(*Set for the first time those rules that will be removed during algebraic transformations*)
Module[{unprotected},
	unprotected=Unprotect[{ qe`plus }];
	qi`$notInAlgebraRules/.{RuleDelayed:>SetDelayed, Rule:>Set};
	Apply[Protect,unprotected];
];

Scan[Function[lista,
With[{qetransformation=lista[[1]],
		transformationlist=lista[[2]],
		unprotectedinsidetransf=lista[[3]]},
	Module[{unprotected},
		unprotected=Unprotect[qetransformation];
		
		(* qe`unknownQ remains unevaluated in qetransformation *)
		qetransformation[arg_?(Or[qe`nonscalarQ[#],qe`scalarQ[#]]&)]:=
		Module[{wereprotected,transformed,qedvals},
			wereprotected=Unprotect[unprotectedinsidetransf];
			(* Save downvalues of unprotected qecommands to restore them at the end: *)
			qedvals=Map[{#,DownValues[#]}&, unprotectedinsidetransf];
			CheckAbort[
				(*remove definitions that would slowdown algebraic transformations: *)
				qi`$notInAlgebraRules/.{RuleDelayed[arg1_,args__]:>Unset[arg1], 
							Rule[arg1_,args__]:>Unset[arg1]};
				(*add definitions that transform: *)
				transformationlist/.{RuleDelayed:>SetDelayed, Rule:>Set};
				transformed=arg; (*transform because of the new definitions *)
				(*remove definitions that transform *)
				transformationlist/.{RuleDelayed[arg1_,args__]:>Unset[arg1], 
							Rule[arg1_,args__]:>Unset[arg1]},
				(* In case of abort: *)
				Print["Restoring default algebra rules"];
				Scan[With[{sym=#[[1]],dvals=#[[2]]},DownValues[sym]=dvals]&, qedvals];
				qi`$notInAlgebraRules/.{RuleDelayed:>SetDelayed, Rule:>Set};
				Apply[Protect,wereprotected];
				Print["Default algebra rules were restored"]
				Abort[] (* do abort after restoring*)
			];
			(*restore rules that were or could have been removed: *)
			Scan[With[{sym=#[[1]],dvals=#[[2]]},DownValues[sym]=dvals]&, qedvals];
			(* next must be the last restore, so those rules can be recovered
			in any case  *)		
			qi`$notInAlgebraRules/.{RuleDelayed:>SetDelayed, Rule:>Set};
			Apply[Protect,wereprotected];
			transformed ];(* return transformed argument *)
			
		Apply[Protect,unprotected]
	]]],
{
{qe`evaluatesymmetrized,qi`$symmevalRules,{qe`plus, qe`symmetric}},
{qe`expandcommutators,qi`$commexpandRules,{qe`plus, qe`qmutator}},
{qe`evaluatecommutators,qi`$commevalRules,{qe`plus, qe`qmutator}},
{qe`arrangeProducts,qi`$arrangeProductsRules,{qe`plus, qe`times}},
{qe`reverseProducts,qi`$reverseProductsRules,{qe`plus, qe`times}},
{qe`arrangePowers,qi`$arrangeRules,{qe`plus, qe`times}},
{qe`unfoldPowers,qi`$unfoldRules,{qe`plus, qe`times,qe`power}},
{qe`collectFromLeft, qi`$leftcollectRules,{qe`plus}},
{qe`collectFromRight, qi`$rightcollectRules,{qe`plus}},
{qe`expandProducts, qi`$expandProductsRules,{qe`plus, qe`times,qe`power}}
} ];


(* arrange products using qmutators instead of commutators *)
qe`arrangeProducts[arg_,q_]:=
	Block[ {qi`qvalue=q}, qe`arrangeProducts[arg] ];
	
qe`reverseProducts[arg_,q_]:=
	Block[ {qi`qvalue=q}, qe`reverseProducts[arg] ];


(* qe`simplify of qe`unknownQ remains unevaluated *)		
qe`simplify[arg_?(Or[qe`nonscalarQ[#],qe`scalarQ[#]]&),otherargs___]:=
	Simplify[arg,otherargs,TransformationFunctions->qi`$simplifyFunctions];

(* qe`fullsimplify of qe`unknownQ remains unevaluated *)				
qe`fullsimplify[arg_?(Or[qe`nonscalarQ[#],qe`scalarQ[#]]&),otherargs___]:=
	FullSimplify[arg,otherargs,TransformationFunctions->qi`$simplifyFunctions];


(* Finishing this package *)


qi`setUnlockedCommands[
	qm`Dirac`qmEvaluateSymmetrized, qe`evaluatesymmetrized,
	qm`Dirac`qmExpandCommutators,qe`expandcommutators,
	qm`Dirac`qmEvaluateCommutators, qe`evaluatecommutators,
	qm`Dirac`qmArrangeProducts, qe`arrangeProducts,
	qm`Dirac`qmReverseProducts, qe`reverseProducts,
	qm`Dirac`qmArrangePowers, qe`arrangePowers, 
	qm`Dirac`qmUnfoldPowers, qe`unfoldPowers,
	qm`Dirac`qmCollectFromLeft,qe`collectFromLeft,
	qm`Dirac`qmCollectFromRight,qe`collectFromRight,
	qm`Dirac`qmExpandProducts, qe`expandProducts,
	qm`Dirac`qmSimplify, qe`simplify,
	qm`Dirac`qmFullSimplify, qe`fullsimplify,
	qe`fullform
];


qi`setPaletteTab["Algebra",3,
			{
				PasteButton[Defer[qm`Dirac`qm[\!\(\*
TagBox[
FrameBox["\<\"expression\"\>"],
"SelectionPlaceholder"]\)]]],
				PasteButton[Defer[qm`Dirac`qmSimplify[\[SelectionPlaceholder]]]],
				PasteButton[Defer[qm`Dirac`qmFullSimplify[\[SelectionPlaceholder]]]],
				
				PasteButton[Defer[qm`Dirac`qmExpandProducts[\[SelectionPlaceholder]]]],
				PasteButton[Defer[qm`Dirac`qmCollectFromLeft[\[SelectionPlaceholder]]]],
				PasteButton[Defer[qm`Dirac`qmCollectFromRight[\[SelectionPlaceholder]]]],

				PasteButton[Defer[qm`Dirac`qmExpandCommutators[\[SelectionPlaceholder]]]],
				PasteButton[Defer[qm`Dirac`qmArrangePowers[\[SelectionPlaceholder]]]], 
				PasteButton[Defer[qm`Dirac`qmUnfoldPowers[\[SelectionPlaceholder]]]],
								
				PasteButton[Defer[qm`Dirac`qmEvaluateCommutators[\[SelectionPlaceholder]]]],											
				PasteButton[Defer[qm`Dirac`qmArrangeProducts[\[SelectionPlaceholder],1]]], 
				PasteButton[Defer[qm`Dirac`qmReverseProducts[\[SelectionPlaceholder],1]]],
				
				(*PasteButton[Defer[qm`Dirac`qmEvaluateSymmetrized[\[SelectionPlaceholder]]]],*)
				
				PasteButton[Defer[qm`Dirac`qmScalarQ[\[SelectionPlaceholder]]]],
				PasteButton[Defer[qm`Dirac`qmNonScalarQ[\[SelectionPlaceholder]]]],
				PasteButton[Defer[qm`Dirac`qmUnknownQ[\[SelectionPlaceholder]]]]
			}
]


(* qe`qmutator must be unprotected so that
the end-user can make definitions, for example
regarding the harmonic oscillator 
commutation relation 
 *)
Unprotect[qe`qmutator]


End[];


qi`message=qi`hello[];


EndPackage[];

Column[{qi`message, qi`sessionAliases[]}]


(*
(* CODE USED TO GENERATE COLLECTFROMLEFT TRANSFORMATIONS *)
(* Later I manually changed one common_ with common_?qe`nonscalarQ in each rule *)
patternfirst=
{Hold[common_,
1],
Hold[qe`power[common_,exponfirst_?IntegerQ],
qe`power[common,exponfirst-1]],
Hold[qe`times[scafirst:_?qe`scalarQ:1,common_,timesfirst___],
qe`times[scafirst,timesfirst]],
Hold[qe`times[scafirst:_?qe`scalarQ:1,qe`power[common_,exponfirst_?IntegerQ],timesfirst___],
qe`times[scafirst,qe`power[common,exponfirst-1],timesfirst]]
};
patternsecond=
{Hold[common_,
1],
Hold[qe`power[common_,exponsecond_?IntegerQ],
qe`power[common,exponsecond-1]],
Hold[qe`times[scasecond:_?qe`scalarQ:1,common_,timessecond___],
qe`times[scasecond,timessecond]],
Hold[qe`times[scasecond:_?qe`scalarQ:1,qe`power[common_,exponsecond_?IntegerQ],timessecond___],
qe`times[scasecond,qe`power[common,exponsecond-1],timessecond]]
};
parejas=Flatten[Table[{patternfirst[[first]],patternsecond[[second]]},{first,Length[patternfirst]},{second,Length[patternsecond]}],1];
reglas=
Block[{qe`times,qe`power,qe`plus},
Map[
Function[lista,
With[{
a11=lista[[1,1]],
a21=lista[[2,1]],
a12=lista[[1,2]],
a22=lista[[2,2]]
},

HoldPattern[
		qe`plus[plusbefore___,
			a11,
			plusbetween___,
			a21,
			plusafter___
		]]:>qe`plus[plusbefore,
				plusbetween,
				qe`times[common,qe`plus[a12,a22]],
				plusafter ]
]

],
parejas]]
*)


(*
(* CODE USED TO GENERATE COLLECTFROMRIGHT TRANSFORMATIONS *)
patternfirst=
{Hold[common_,
1],
Hold[qe`power[common_,exponfirst_?IntegerQ],
qe`power[common,exponfirst-1]],
Hold[qe`times[timesfirst___,common_],
qe`times[timesfirst]],
Hold[qe`times[timesfirst___,qe`power[common_,exponfirst_?IntegerQ]],
qe`times[timesfirst,qe`power[common,exponfirst-1]]]
};
patternsecond=
{Hold[common_,
1],
Hold[qe`power[common_,exponsecond_?IntegerQ],
qe`power[common,exponsecond-1]],
Hold[qe`times[timessecond___,common_],
qe`times[timessecond]],
Hold[qe`times[timessecond___,qe`power[common_,exponsecond_?IntegerQ]],
qe`times[timessecond,qe`power[common,exponsecond-1]]]
};
parejas=Flatten[Table[{patternfirst[[first]],patternsecond[[second]]},{first,Length[patternfirst]},{second,Length[patternsecond]}],1];
reglas=
Block[{qe`times,qe`power,qe`plus},
Map[
Function[lista,
With[{
a11=lista[[1,1]],
a21=lista[[2,1]],
a12=lista[[1,2]],
a22=lista[[2,2]]
},

HoldPattern[
		qe`plus[plusbefore___,
			a11,
			plusbetween___,
			a21,
			plusafter___
		]]:>qe`plus[plusbefore,
				plusbetween,
				qe`times[qe`plus[a12,a22],common],
				plusafter ]
]

],
parejas]]
*)
