(* ::Package:: *)

General::qmfatal="The new Quantum packages and the old Quantum packages are NOT compatible.";
If[TrueQ[MemberQ[$Packages,"Quantum`Notation`"]],
	Print["The new Quantum packages and the old Quantum packages are NOT compatible."];
	Message[General::qmfatal];
	Abort[];
];


BeginPackage["qm`qmBasic`",{"Global`"}];


 (* 
   Copyright Jose Luis Gomez-Munoz and Francisco Delgado
   ITESM-CEM, 
   Departamento de F\[IAcute]sica y Matem\[AAcute]ticas
   Carretera Lago de Guadalupe Km. 3.5, 
   Atizap\[AAcute]n de Zaragoza, Estado de M\[EAcute]xico, 
   C.P. 52926
   Mexico
   jose.luis.gomez@itesm.mx
   http://homepage.cem.itesm.mx/jose.luis.gomez
*)


Unprotect[qi`$basicNames];
ClearAll[qi`$basicNames];
qi`$basicNames=HoldComplete[
			(* commands *)
			qm`Dirac`qm,
			qm`Dirac`qmFullForm,
			qm`Dirac`qmPalette,
			qm`Dirac`qmSessionAliases,
			(* information lists *)
			qi`$modifiedSymbols, 
			qi`$impossibleSymbols,
			qi`$commands,
			(* programmer commands *)
			qi`notebookAliases,
			qi`outsideqm,
			qi`hello,
			(* programmer commands, append to data lists: *)
			qi`setPaletteTab,
			qi`setPreRules,
			qi`setPostRules,
			qi`setAlias,
			(* programmer commands, append to information lists *)
			qi`setCommands,
			qi`setUnlockedCommands,
			qi`setModifiedSymbols,
			qi`setClearAllSymbols,
			qi`setImpossibleSymbols,
			(* programmer information data: *)
			qi`$version,
			(* programmer data lists *)	
			qi`$paletteTabs,
			qi`$preRules,
			qi`$postRules,
			qi`$aliases,
			(* programmer package parameters: *)
			qi`$verify,
			qi`$initialPaletteTabIndex,
			qi`$paletteNotebook
];
Apply[Unprotect,qi`$basicNames];
Apply[ClearAll,qi`$basicNames];
Scan[Function[#::usage=ToString[#]<>" is a symbol of the qm package"],
	qi`$basicNames];
(* Next command does NOT protect the names (commands) inside the list qi`$basicNames;
   it DOES protect the name of the list itself *)
SetAttributes[qi`$basicNames,{Protected,ReadProtected}];
qi`$basicNames::usage="qi`$basicNames is an internal symbol of the qm package";
qi`logPrint::usage="qi`logPrint=Print in order to print debug info of qm packages";


(*These list should be modified only with the qi`set* commands *)
(* Informtation lists *)
qi`$modifiedSymbols ={};
qi`$impossibleSymbols={};
qi`$commands={};
(* Data lists *)
qi`$paletteTabs={};
qi`$preRules={};
qi`$postRules={};
qi`$aliases={};

(*These lists are protected, they should be modified only with qi`set* commands *)
Protect[
(* Informtation lists *)
		qi`$modifiedSymbols, 
		qi`$impossibleSymbols,
		qi`$commands,
(* Data lists *)
		qi`$paletteTabs,
		qi`$preRules,
		qi`$postRules,
		qi`$aliases];
(* Data lists are read-protected. Information lists are not*)
SetAttributes[{
		qi`$paletteTabs,
		qi`$preRules,
		qi`$postRules,
		qi`$aliases
	},
	ReadProtected
];


SetAttributes[{
		qm`Dirac`qm,
		qm`Dirac`qmFullForm,
		qi`setModifiedSymbols,
		qi`setClearAllSymbols,
		qi`setImpossibleSymbols
	},
	HoldAllComplete
];


Begin["`Private`"];


qm`Dirac`qm[one_,more__]:=
(Message[qm`Dirac`qm::argx,qm`Dirac`qm,1+Length[Unevaluated[{more}]]];Abort[]);

(* The flag qi`outsideqm is used so that the output of qm[arg]
is also wrapped by qm, like qm[result], without infinite loop.
Block restores the value of the flag to true after evaluation *)
qi`outsideqm=True;
Protect[qi`outsideqm];
HoldPattern[qm`Dirac`qm[arg_]/;qi`outsideqm]:=
Block[{qi`outsideqm=False,qi`qmarg},
		qi`logPrint["qm"," enter qm"];
		qi`qmarg=HoldComplete[arg];
		qi`logPrint["qm"," qm1 ",qi`qmarg];
		qi`qmarg=qi`qmarg//.qi`$preRules;
		qi`logPrint["qm"," qm2 ",qi`qmarg];
		(* evaluate and handle nested qm from use of % *)
		qi`qmarg=ReleaseHold[qi`qmarg]//. qm`Dirac`qm->qm`Dirac`qmFullForm; 
		qi`logPrint["qm"," qm3 ",qi`qmarg];
		With[{replace=qi`qmarg},qi`qmarg=HoldComplete[replace]];
		qi`logPrint["qm"," qm4 ",qi`qmarg];
		qi`qmarg=qi`qmarg//.qi`$postRules;
		qi`logPrint["qm"," qm5 ",qi`qmarg];
		Apply[qm`Dirac`qm,qi`qmarg]
];

(* It is very important that qm`Dirac`qmFullForm has attribute HoldAllComplete. 
it must also be declared as undefined (not scalar and not nonscalar),
in march 2017 this is done in the file qmSlotsExpressions.m
qm`Dirac`qmFullForm is generated from nested qm[qm[]...], which in turn can
be generated (injected) by the use of % in order to use the previous output *)
HoldPattern[qm`Dirac`qmFullForm[arg_]]:=
Block[{qi`qmarg},
		qi`logPrint["qeform"," enter qeform"];
		qi`qmarg=HoldComplete[arg];
		qi`logPrint["qeform"," qeform1 ",qi`qmarg];
		qi`qmarg=qi`qmarg//.qi`$preRules;
		qi`logPrint["qeform"," qeform2 ",qi`qmarg];
		ReleaseHold[qi`qmarg]
];



qi`hello[]:=With[{modlist=Join[qi`$impossibleSymbols,qi`$modifiedSymbols]},
Column[
Join[
	{"The qm add-on "<>Context[]<>" version "<>qi`$version<>" has been loaded today "<>DateString[],
		"\!\(\*
StyleBox[\"Mathematica\",\nFontSlant->\"Italic\"]\) "<>$Version},
	If[Length[modlist]>0,
		{"Symbols that have been modified by loading the qm add-ons:"},
		{}],
	If[Length[modlist]>0,
		{Grid[Partition[PadRight[modlist/.HoldComplete->HoldForm,
							Length[modlist]+3," "],4],Dividers->All]},
		{}]
]]]


(* The correct working of the palettes, at least the way I am implementing
them, requires the PlaceHolder and SelectedPlaceHolder to be Global variables *)
Global`\[Placeholder];
Global`\[SelectionPlaceholder];

qi`setPaletteTab[tablabel_,partition_,buttonlist_]:=
Module[{posinfo, copiatabs},
	Unprotect[qi`$paletteTabs];
	posinfo=Position[qi`$paletteTabs,tablabel];
	If[posinfo==={},
		AppendTo[qi`$paletteTabs,{tablabel,partition,buttonlist}],
	(*else*)
		copiatabs=qi`$paletteTabs;
		copiatabs[[ posinfo[[1,1]],3 ]]=
			Join[copiatabs[[ posinfo[[1,1]],3 ]], buttonlist ];
		qi`$paletteTabs=copiatabs
	];
	Protect[qi`$paletteTabs];
];

qm`Dirac`qmPalette[]:=qm`Dirac`qmPalette[qi`$paletteTabs];

qm`Dirac`qmPalette[paletteTabs_]:=
Module[{tv},
	tv=TabView[
		Map[
			Function[record,
				{record[[1]],record[[1]]->
					Grid[Partition[record[[3]],record[[2]]],Dividers->All]}],
			paletteTabs],
		qi`$initialPaletteTabIndex,
		Alignment->Center,
		ImageSize->Automatic];
	If[MemberQ[Notebooks[],qi`$paletteNotebook],NotebookClose[qi`$paletteNotebook]];
	Unprotect[qi`$paletteNotebook];
	qi`$paletteNotebook=
		CreatePalette[tv,WindowTitle->"Quantum Mechanics",Saveable->False,
						Magnification->1.25];
	Protect[qi`$paletteNotebook];
	qi`$paletteNotebook
];


qi`setAlias[alias_String,boxes_,message_]:=
Module[{},
	Unprotect[qi`$aliases];
	AppendTo[qi`$aliases,{alias,boxes,message}];
	Protect[qi`$aliases];
]; 

qi`notebookAliases[]:=qi`notebookAliases[InputNotebook[]];

qi`notebookAliases[doc_NotebookObject]:=
Module[{aliases,unionaliases,mensajes,nbfn,error},
	aliases=Function[{a,b},a->b]@@@qi`$aliases[[All,1;;2]];
	unionaliases=Union[aliases,Part[Options[doc,InputAliases],1,2]];
	SetOptions[doc,InputAliases->unionaliases];
	mensajes=Function[{a,b},{"\[EscapeKey]"<>a<>"\[EscapeKey]",b}]@@@qi`$aliases[[All,1;;3;;2]];
	nbfn=Quiet[Check[NotebookFileName[doc],error]];
	If[nbfn=!=error,
		PrependTo[mensajes,
		{Row[{"New keyboard aliases for notebook: ",
			Last[FileNameSplit[nbfn]]}],SpanFromLeft}],
		PrependTo[mensajes,
		{Row[{"New keyboard aliases for notebook: ",doc}],SpanFromLeft}]
	];
	AppendTo[mensajes,
		{"These aliases begin and end with the [ESC] key",SpanFromLeft}];
	Grid[mensajes,
		Alignment->Left,Dividers->All]
];

qm`Dirac`qmSessionAliases[]:=
Module[{aliases,unionaliases,mensajes,nbfn,error},
	aliases=Function[{a,b},a->b]@@@qi`$aliases[[All,1;;2]];
	unionaliases=Union[aliases,Part[Options[$FrontEndSession,InputAliases],1,2]];
	SetOptions[$FrontEndSession,InputAliases->unionaliases];
	mensajes=Function[{a,b},{"\[EscapeKey]"<>a<>"\[EscapeKey]",b}]@@@qi`$aliases[[All,1;;3;;2]];
	PrependTo[mensajes,
		{"New keyboard aliases for this Mathematica session",SpanFromLeft}];
	AppendTo[mensajes,
		{"These aliases begin and end with the [ESC] key",SpanFromLeft}];
	Grid[mensajes,
		Alignment->Left,Dividers->All]
];


qi`setModifiedSymbols[{args__}]:=qi`setModifiedSymbols[args];

qi`setModifiedSymbols[args__]:=
	Scan[qi`setModifiedSymbols,Unevaluated[{args}]] /; 
		Length[Unevaluated[{args}]]>1;

qi`setModifiedSymbols[{}]:=Null;

qi`setModifiedSymbols[expr_]:=
Module[{unprotected},
	unprotected=Unprotect[expr,qi`$modifiedSymbols];
	expr::usage=
		ToString[expr]<>
		" has been modified by the Quantum Mechanics qm packages";
	qi`$modifiedSymbols=Union[qi`$modifiedSymbols,{HoldComplete[expr]}];
	qi`logPrint["Registered as modified symbol ",HoldComplete[expr]];
	Protect[Evaluate[unprotected]]
];


qi`setClearAllSymbols[{args__}]:=qi`setClearAllSymbols[args];

qi`setClearAllSymbols[args__]:=
	Scan[qi`setClearAllSymbols,Unevaluated[{args}]] /; 
		Length[Unevaluated[{args}]]>1;

qi`setClearAllSymbols[{}]:=Null;

qi`setClearAllSymbols[expr_]:=
Module[{},
	qi`logPrint["ClearAll and protecting: ",HoldComplete[expr]];
	Unprotect[expr];
	ClearAll[expr];
	expr::usage=
		SymbolName[expr]<>
		" must be used inside an expression that is wrapped with qm[ ]"<>
		" in order to obtain correct results from Quantum Mechanics calculations";
	HoldPattern[expr[args___]]:=
		With[{e=expr},
			(Message[e::qmunknown,HoldForm[e]];Abort[])/;
				(If[And[qi`$verify,MemberQ[$Packages,"Quantum`Notation`"]],
					(Message[e::qmfatal];Abort[])];
				If[qi`$verify,(Message[e::usage];Abort[])];
				False)];  
	Protect[expr];
	SetAttributes[expr,ReadProtected];
	qi`logPrint["END ClearAll and protecting: ",HoldComplete[expr]];
];


qi`setImpossibleSymbols[{args__}]:=qi`setImpossibleSymbols[args];

qi`setImpossibleSymbols[args__]:=
	Scan[qi`setImpossibleSymbols,Unevaluated[{args}]] /; 
		Length[Unevaluated[{args}]]>1;

qi`setImpossibleSymbols[{}]:=Null;

qi`setImpossibleSymbols[expr_]:=
Module[{},
	qi`logPrint["Impossible Symbol: ",HoldComplete[expr]];
	Unprotect[expr];
	ClearAll[expr];
	expr::usage=
		"Symbol "<>ToString[HoldForm[expr]]<>" cannot be used as a variable";
	expr::qmfatal=
		"The new qm packages and the old Quantum packages are NOT compatible."<>
		" Please Quit the kernel or Quit Mathematica and start a new session";
	HoldPattern[expr]:=
		(Message[expr::qmunknown,HoldForm[expr]];Abort[])/;
				(If[And[qi`$verify,MemberQ[$Packages,"Quantum`Notation`"]],
					(Message[expr::qmfatal];Abort[])];
				If[qi`$verify,
					(Message[expr::usage];Abort[])];
				False);  
	Protect[expr];
	SetAttributes[expr,ReadProtected];
	Unprotect[qi`$impossibleSymbols];
	qi`$impossibleSymbols=Union[qi`$impossibleSymbols,{HoldComplete[expr]}];
	Protect[qi`$impossibleSymbols];	
	qi`logPrint["END Impossible Symbol: ",HoldComplete[expr]];
]


qi`setPreRules[{args__}]:=qi`setPreRules[args];

qi`setPreRules[args__]:=
	Scan[qi`setPreRules,{args}] /; Length[{args}]>1;

qi`setPreRules[{}]:=Null;

qi`setPreRules[rule_]:=
Module[{},
	Unprotect[qi`$preRules];
	AppendTo[qi`$preRules,rule];
	Protect[qi`$preRules];
]; 


qi`setPostRules[{args__}]:=qi`setPostRules[args];

qi`setPostRules[args__]:=
	Scan[qi`setPostRules,{args}] /; Length[{args}]>1;

qi`setPostRules[{}]:=Null;

qi`setPostRules[rule_]:=
Module[{},
	Unprotect[qi`$postRules];
	AppendTo[qi`$postRules,rule];
	Protect[qi`$postRules];
]; 


(* Commands are Protected and ReadProtected *)

qi`setCommands[{args__}]:=qi`setCommands[args];

qi`setCommands[args__]:=Scan[qi`setCommands,{args}] /; Length[{args}]>1;

qi`setCommands[{}]:=Null;

qi`setCommands[expr_Symbol]:=
Module[{},
	SetAttributes[expr,{ReadProtected,Protected,Locked}];
	Unprotect[qi`$commands];
	qi`$commands=Union[qi`$commands,{expr}];
	Protect[qi`$commands];
];

qi`setUnlockedCommands[{args__}]:=qi`setUnlockedCommands[args];

qi`setUnlockedCommands[args__]:=
	Scan[qi`setUnlockedCommands,{args}] /; Length[{args}]>1;

qi`setUnlockedCommands[{}]:=Null;

qi`setUnlockedCommands[expr_Symbol]:=
Module[{},
	Unprotect[expr];
	SetAttributes[expr,{ReadProtected,Protected}];
	Unprotect[qi`$commands];
	qi`$commands=Union[qi`$commands,{expr}];
	Protect[qi`$commands];
];


(* Finishing this package *)


qi`$verify=True;


qm`Dirac`qm::usage=
"qm[expression] evaluates expression assuming it contains"<>
" quantum mechanics symbols, such as noncommutative operators";
qi`$modifiedSymbols::usage=
"qi`$modifiedSymbols gives a list of the standard Mathematica symbols which have been cleared by loading the qm packages";
qi`$impossibleSymbols::usage=
"qi`$impossibleSymbols gives a list of the standard Mathematica symbols that cannot be used after loading the qm packages";
qi`notebookAliases::usage=
"qi`notebookAliases[] sets additional [ESC]name[ESC] input-aliases for qm commands in the InputNotebook[]\n"<> 
"qi`notebookAliases[notebook] sets additional [ESC]name[ESC] input-aliases for notebook";
qm`Dirac`qmSessionAliases::usage=
"qmSessionAliases[] sets additional [ESC]name[ESC] input-aliases for qm commands in the current session";
qm`Dirac`qmPalette::usage=
"qmPalette[] opens the palette (toolbar) for qm commands and templates";


qi`$preRules::qmreplace="qm: Replacing `1` with `2`.";
qi`$postRules::qmreplace="qm: Replacing `1` with `2`.";

General::qmunknown="Unknown internal error in the Quantum Mechanincs qm packages `1`";


qi`setCommands[
	qi`notebookAliases, qi`setAlias,
	qi`setPaletteTab,  
	qi`setClearAllSymbols,
	qi`setModifiedSymbols,
	qi`setImpossibleSymbols,
	qi`setCommands, 
	qi`setUnlockedCommands,
	qi`hello,
	qi`setPreRules,
	qi`setPostRules
];


qi`setUnlockedCommands[
	qm`Dirac`qm, qm`Dirac`qmSessionAliases, qm`Dirac`qmPalette,
	qm`Dirac`qmFullForm
];


End[];


qi`$version="qmBasic.m";
qi`message=qi`hello[];


EndPackage[];

qi`message
