(* ::Package:: *)

BeginPackage["qm`qmBraKet`",
{"qm`qmBasic`","qm`qmTypesetting`",
"qm`qmSlots`","qm`qmSlotsPrimitives`","qm`qmSlotsExpressions`",
"qm`qmOperations`",
"Global`"}];


(* 
   Copyright 2016
   Jose Luis Gomez-Munoz and Francisco Delgado
   ITESM-CEM, 
   Departamento de F\[IAcute]sica y Matem\[AAcute]ticas (Ciencias B\[AAcute]sicas)
   Carretera Lago de Guadalupe Km. 3.5, 
   Atizap\[AAcute]n de Zaragoza, Estado de M\[EAcute]xico, 
   C.P. 52926
   Mexico
   jose.luis.gomez@itesm.mx
   http://homepage.cem.itesm.mx/jose.luis.gomez/
*)


Begin["`Private`"];


unprotected=Unprotect[qe`ket,qe`bra,qe`brkt,qe`hermitianConjugate,qe`subscript];


qe`ket::qmnotp="Could not calculate tensor product of kets `1`, `2`";
qe`ket::qmsspa="Kets `1`, `2` cannot be multiplied because they share state space `3`";
qe`bra::qmnotp="Could not calculate tensor product of bras `1`, `2`";
qe`bra::qmsspa="Bras `1`, `2` cannot be multiplied because they share state space `3`";


(* times[bra,ket] evolves to times[ket,braket,bra]. 
This is the main, most important product rule for kets and bras
that have well-defined slot labels (see condition at the end of the
definition, after /;).
*)

qe`ket /: HoldPattern[qe`times[
	before___,
	bra:qe`bra[braargs__],
	ket:qe`ket[ketargs__],
	after___
]]:=
With[{braslots=qi`slotlabels[bra],ketslots=qi`slotlabels[ket]},
	Module[{newbralabels,newketlabels,brabklabels,ketbklabels,
			newbra,newbk,newket},
		newbralabels=Complement[braslots[[2]],ketslots[[1]],SameTest->(First[#1]==First[#2]&)];
		brabklabels=Complement[braslots[[2]],newbralabels,SameTest->(First[#1]==First[#2]&)];
		newketlabels=Complement[ketslots[[1]],braslots[[2]],SameTest->(First[#1]==First[#2]&)];
		ketbklabels=Complement[ketslots[[1]],newketlabels,SameTest->(First[#1]==First[#2]&)];
		If[TrueQ[newbralabels==={}], 
				newbra=Sequence[],
				newbra=qe`bra@@Map[qe`subscript[#[[2]],#[[1]]]&,newbralabels]];
		If[TrueQ[newketlabels==={}], 
				newket=Sequence[],
				newket=qe`ket@@Map[qe`subscript[#[[2]],#[[1]]]&,newketlabels]];
		If[And[brabklabels==={},ketbklabels==={}], 
				newbk=Sequence[],
				newbk=qe`brkt[Map[qe`subscript[#[[2]],#[[1]]]&,brabklabels],
								Map[qe`subscript[#[[2]],#[[1]]]&,ketbklabels]]];
		qe`times[
			before,
			newket,
			newbk,
			newbra,
			after]]
	/; And[FreeQ[braslots,Missing],FreeQ[ketslots,Missing]]
];


(* unbalanced braket becomes bra ket, see the condition
at the end of the defintion, branumbers[[2]]=!=ketnumbers[[1]]
*)
HoldPattern[qe`brkt[{braargs___},{ketargs___}]]:=
With[{branumbers=qi`slotnumbers[qe`bra[braargs]],
		ketnumbers=qi`slotnumbers[qe`ket[ketargs]]},
	qe`times[qe`bra[braargs],qe`ket[ketargs]]
	/; And[FreeQ[branumbers,Missing],
			FreeQ[ketnumbers,Missing],
			branumbers[[2]]=!=ketnumbers[[1]]]
];


qe`ket /: HoldPattern[qe`hermitianConjugate[qe`ket[args___]]]:=qe`bra[args];
qe`bra /: HoldPattern[qe`hermitianConjugate[qe`bra[args___]]]:=qe`ket[args];
qe`brkt /: HoldPattern[qe`conjugate[qe`brkt[{braargs___},{ketargs___}]]]:=qe`brkt[{ketargs},{braargs}];


HoldPattern[qe`subscript[qe`bra[k__],subs__]]:=
	qe`hermitianConjugate[qe`subscript[qe`ket[k],subs]];


(* Finishing this package *)


Protect[Evaluate[unprotected]];
(* qe`brkt must be unprotected so that
the end-user can make definitions, for example
regarding orthogonlaity. Furthermore, 
qe`subscript must be also unprotected so that
end user can define subscripted kets:
 *)
Unprotect[qe`brkt, qe`subscript]
End[];


qi`message=qi`hello[];


EndPackage[];

Column[{qi`message, qi`sessionAliases[]}]
