(* ::Package:: *)

BeginPackage["qm`qmOperations`",
{"qm`qmBasic`","qm`qmTypesetting`",
"qm`qmSlots`","qm`qmSlotsPrimitives`","qm`qmSlotsExpressions`",
"Global`"}];


(* 
   Copyright 2017
   Jose Luis Gomez-Munoz and Francisco Delgado
   ITESM-CEM, 
   Departamento de F\[IAcute]sica y Matem\[AAcute]ticas (Ciencias B\[AAcute]sicas)
   Carretera Lago de Guadalupe Km. 3.5, 
   Atizap\[AAcute]n de Zaragoza, Estado de M\[EAcute]xico, 
   C.P. 52926
   Mexico
   jose.luis.gomez@itesm.mx
   http://homepage.cem.itesm.mx/jose.luis.gomez/
*)


Unprotect[qi`$operationsNames];
ClearAll[qi`$operationsNames];
qi`$operationsNames=HoldComplete[
			(* programmer commands *)
			qi`freeHermitianQ,
			qi`reverseOrderingFlag,
			qi`qmutatorOutsideFlag,
			qi`normalOrderQ,
			(* programmer lists *)
			qi`$qobjects,
			qi`$scalarFunctions
];
Apply[Unprotect,qi`$operationsNames];
Apply[ClearAll,qi`$operationsNames];
Scan[Function[#::usage=ToString[#]<>" is a symbol of the Quantum Mechanics packages"],
	qi`$operationsNames];
(* Next command does NOT protect the names (symbols) inside the list qi`$operationsNames;
   it DOES protect the name of the list itself *)
SetAttributes[qi`$operationsNames,{Protected,ReadProtected}];
qi`$operationsNames::usage="qi`$operationsNames is an internal symbol of the Quantum Mechanics packages";


(* I am NOT giving to qe`plus, qe`times and qe`power 
the attributes of Plus, Times and Power because
those attributes (Flat, Orderless, OneIdentity) can produce
very inefficient execution time if naive programming techniques are used,
and I prefer to use those naive and simple programming techniques, 
therefore I will NOT set those Attributes to my commands *)


qi`$qobjects=
	Join[qi`$numfunprerules[[All,2]],
		{qe`ket,qe`bra,qe`opeHat,qe`qmutator,qe`hermitianConjugate}];
qi`$scalarFunctions=
	Join[qi`$numfunprerules[[All,1]],
		{DiracDelta, HeavisideTheta, DirectedInfinity}];
Protect[qi`$qobjects,qi`$scalarFunctions];


Begin["`Private`"];


(* commands for Normal Ordering *)

qi`freeHermitianQ[expr_]:=
	And[FreeQ[expr,qe`hermitianConjugate],FreeQ[expr,qi`dagger]];

qi`reverseOrderingFlag=False;
qi`normalOrderQ[first_,second_]:=
	Module[ {answer,firstbase,secondbase},
		answer=Which[
		And[Not[FreeQ[first,qe`qmutator]],FreeQ[second,qe`qmutator]],True,
		And[FreeQ[first,qe`qmutator],Not[FreeQ[second,qe`qmutator]]],False,
		And[Not[qi`freeHermitianQ[first]],qi`freeHermitianQ[second]],True,
		And[qi`freeHermitianQ[first],Not[qi`freeHermitianQ[second]]],False,
		True,
			(If[Head[first]===qe`power,
				firstbase=first[[1]],
				firstbase=first];
			If[Head[second]===qe`power,
				secondbase=second[[1]],
				secondbase=second];
			OrderedQ[{firstbase,secondbase}]) ];
	If[TrueQ[qi`reverseOrderingFlag],answer=Not[answer]];
	answer];


unprotected=Union[
		Unprotect[Evaluate[qi`$qobjects]],
		Unprotect[qe`qmutator, qe`symmetric,
		qe`equal,qe`expectation,qe`hermitianConjugate]];


(* The quantum objects,
when they are determined to be nonscalars, 
cannot be arguments of numeric functions,
(it must be the qe equivalent, qe`exp instead of Exp, etc.)
they will Abort[] *)

General::qmnumf="Noncommutative object `1` cannot be an argument of commutative function `2`";
Do[
	With[{head=qi`$scalarFunctions[[nf]]},
		Map[
			Which[
				head===Power,
					((#/:HoldPattern[Power[base_,Pattern[qexpr,Blank[#]]]/;
									qe`nonscalarQ[qexpr]]:=
					(Message[#::qmnumf, qexpr, Power]; Abort[]));
					(#/:HoldPattern[Power[Pattern[qexpr,Blank[#]],expo_]/;
									qe`nonscalarQ[qexpr]]:=
					(Message[#::qmnumf, qexpr, Power]; Abort[]))),
				MemberQ[Attributes[head],OneIdentity],
					(#/:HoldPattern[head[Pattern[qexpr,Blank[#]]]/;
									qe`nonscalarQ[qexpr]]:=
					(Message[#::qmnumf, qexpr, head]; Abort[])),
				True,
					(#/:HoldPattern[head[left___,Pattern[qexpr,Blank[#]],right___]/;
									qe`nonscalarQ[qexpr]]:=
					(Message[#::qmnumf, qexpr, head]; Abort[]))]&,
			qi`$qobjects]],
{nf,1,Length[qi`$scalarFunctions]}]; 



(* The quantum objects,
when they are determined to be nonscalars, 
cannot be the first argument of 
standard Mathematica iterated operations,
(it must be the qe equivalent, qe`sum instead of Sum, etc.)
they will Abort[] *)
General::qmscalarf="Noncommutative object `1` cannot be the first argument of commutative operation `2`";
Do[
	With[{head=cabeza},
		Map[(#/:HoldPattern[head[Pattern[qexpr,Blank[#]],args__]/;qe`nonscalarQ[qexpr]]:=
				(Message[#::qmscalarf, qexpr, head]; Abort[]))&,
			qi`$qobjects]],
{cabeza,{Sum,NSum,Integrate,NIntegrate,Product,NProduct}}];


(* If the arguments of the quantum equivalents qe of numeric functions
are all scalars, the qe function is transformed back to the numeric function *)
Do[With[{qefun=qi`$numfunprerules[[nf,2]],fun=qi`$numfunprerules[[nf,1]]},
			HoldPattern[qefun[args:((_?qe`scalarQ)..)]]:=fun[args]],
	{nf,1,Length[qi`$numfunprerules]}];


(* Defaults for pattern matching, 
important for those definitions below that use the pattern _. *)
qe`plus/:Default[qe`plus]:=0;
qe`times/:Default[qe`times]:=1;
qe`power/:Default[qe`power,2]:=1;


(* Asociativity *)
HoldPattern[qe`times[args1___,qe`times[args__], args2___] ]:=
	qe`times[args1,args,args2];

HoldPattern[qe`plus[args1___,qe`plus[args__], args2___] ]:=
	qe`plus[args1,args,args2];

(* To make qe`power similar to Power, 
when more than two arguments are specified then we have a "power tower",
this is similar to the asociativity, but applied in the opposite direction *)
HoldPattern[ 
	qe`power[arg1_, arg2_, argmore__] ]:=qe`power[arg1,qe`power[arg2, argmore]];


(* Less than two arguments *)
HoldPattern[qe`times[]]:=1;
HoldPattern[qe`times[x_]]:=x;

HoldPattern[qe`plus[]]:=0;
HoldPattern[qe`plus[x_]]:=x;

HoldPattern[qe`power[]]:=1;
HoldPattern[qe`power[x_]]:=x;


(* scalars to the begining, kind of a limited commutativity *)
HoldPattern[ qe`times[ 
	arg1___,
	argnon_?qe`nonscalarQ,
	argsca_?qe`scalarQ,
	arg2___ ] ]:=
	qe`times[arg1,argsca,argnon,arg2];



(* standard Mathematica commands from qm-commands *)

HoldPattern[qe`expectation[sca_?qe`scalarQ]]:=sca;

HoldPattern[qe`hermitianConjugate[qe`hermitianConjugate[any_]]]:=any;

(* hermitian goes to qe`conjugate instead of Conjugate in order
 for any special rule defined for qe`conjugate to be applied before
 actually reaching the "final" Conjugate *)
HoldPattern[qe`hermitianConjugate[sca_?qe`scalarQ]]:=qe`conjugate[sca];
HoldPattern[qe`conjugate[sca_?qe`scalarQ]]:=Conjugate[sca];
HoldPattern[qe`conjugate[nonsca_?qe`nonscalarQ]]:=
		With[{e=qe`conjugate},
			(Message[e::qmunknown,HoldForm[e]];Abort[])/;
				(If[And[qi`$verify,MemberQ[$Packages,"Quantum`Notation`"]],
					(Message[e::qmfatal];Abort[])];
				If[qi`$verify,Message[e::qmnonscalar,nonsca]];
				False)];
(* assuming scalars automatically go to the begining in qe`times: *)
HoldPattern[ qe`times[
	argsca1_?qe`scalarQ,
	argsca2_?qe`scalarQ,
	args___]]:=
	qe`times[Times[argsca1,argsca2],args];
(*  scalars might be in any position in qe`plus *)
HoldPattern[ qe`plus[
	argsleft___,
	argsca1_?qe`scalarQ,
	argscenter___,
	argsca2_?qe`scalarQ,
	argsright___]]:=
	qe`plus[Plus[argsca1,argsca2],argsleft,argscenter,argsright];

HoldPattern[ qe`power[
	argsca1_?qe`scalarQ,
	argsca2_?qe`scalarQ]]:=
	Power[argsca1,argsca2];

HoldPattern[qe`power[qe`power[base_,n1_?IntegerQ],n2_?IntegerQ]]:=qe`power[base,n1*n2];


(* Identity and other special arguments of operations
  Using 0 vs 0.0 and 1 vs 1.0 same as 
in standard Mathematica Plus, Times and Power *)
HoldPattern[qe`plus[argsleft___,0,argsright___]]:=
	qe`plus[argsleft,argsright];
HoldPattern[qe`times[argsleft___,1,argsright___]]:=
	qe`times[argsleft,argsright];
HoldPattern[qe`times[argsleft___,0,argsright___]]:=0;
HoldPattern[qe`times[argsleft___,0.0,argsright___]]:=0.0;
HoldPattern[qe`power[base_,1]]:=base;
HoldPattern[qe`power[base_,0]]:=1;
HoldPattern[qe`power[base_,0.0]]:=1.0;


qe`plus /: HoldPattern[qe`expectation[qe`plus[args__]]]:=
	Apply[qe`plus,Map[qe`expectation,{args}]];
qe`times /: HoldPattern[qe`expectation[qe`times[sca_?qe`scalarQ,args__]]]:=
	qe`times[sca,qe`expectation[qe`times[args]]];
qe`plus /: HoldPattern[qe`hermitianConjugate[qe`plus[args__]]]:=
	Apply[qe`plus,Map[qe`hermitianConjugate,{args}]];
qe`times /: HoldPattern[qe`hermitianConjugate[qe`times[args__]]]:=
	Apply[qe`times,Map[qe`hermitianConjugate,Reverse[{args}]]];


(* Quantum product of (powers of) consecutive (noncommutative!) identical objects 
goes to power *)
(* In the next rule we must use only one underscore x_, and as it is always in the LHS
inside a power, it can have head times or plus *)
qe`power /: HoldPattern[qe`times[
		args1___, 
		qe`power[x_,m_.],qe`power[x_,n_.], 
		args2___
		]]:= qe`times[args1, qe`power[x,(m+n)], args2] /; 
			And[qe`scalarQ[m],qe`scalarQ[n]];
			
(* Next rule, with one underscore x_, does Not include sequences ab(ab)^2, 
because, as times is automatically flatten, the first ab inside ab(ab)^2 does not have another head times,
but the ab inside (ab)^2 does have head times. Therefore ab(ab)^2 is handled in qmArrangePowers *)
qe`power /: HoldPattern[qe`times[
		args1___, 
		x_,qe`power[x_,n_.], 
		args2___
		]]:= qe`times[args1, qe`power[x,(1+n)], args2] /; 
			qe`scalarQ[n];

(* Next rule, with one underscore x_, does Not include sequences (ab)^2ab, 
because, as times is automatically flatten, the second ab inside (ab)^2ab does not have another head times,
but the ab inside (ab)^2 does have head times. Therefore (ab)^2ab is handled in qmArrangePowers *)
qe`power /: HoldPattern[qe`times[
		args1___, 
		qe`power[x_,m_.],x_, 
		args2___
		]]:= qe`times[args1, qe`power[x,(m+1)], args2] /; 
			qe`scalarQ[m];

(* Next rule is only for one symbol, a*a->a^2
The version for more a*b*a*b->(a*b)^2 is in qmArrangePowers *)
HoldPattern[qe`times[
		args1___, 
		x_,x_, 
		args2___
		]]:= qe`times[args1, qe`power[x,2], args2];

(* Quantum plus of (multiples of) non-consecutive (commutative!) identical objects 
goes to times *)
(* Here xseq__ with two underscores, because xseq appears always 
inside times, both in the lhs and the rhs *)
qe`times/: HoldPattern[qe`plus[
		args1___, 
		qe`times[m_.,xseq__],
		args2___,
		qe`times[n_.,xseq__], 
		args3___
		]]:= qe`plus[args1, args2, args3, qe`times[(m+n),xseq]] /; 
			And[qe`scalarQ[m],qe`scalarQ[n]];

(* Must be only one underscore x_ *)
qe`times/: HoldPattern[qe`plus[
		args1___, 
		x_,
		args2___,
		qe`times[n_.,x_], 
		args3___
		]]:= qe`plus[args1, args2, args3, qe`times[(1+n),x]] /; 
			qe`scalarQ[n];
(* Must be only one underscore x_
This is also included, because I implemented commutativity 
without attributes, therefore the previous one and the next one are both
needed *)
qe`times/: HoldPattern[qe`plus[
		args1___, 
		qe`times[m_.,x_],
		args2___,
		x_, 
		args3___
		]]:= qe`plus[args1, args2, args3, qe`times[(m+1),x]] /; 
			qe`scalarQ[m];

HoldPattern[qe`plus[
		args1___, 
		xseq_,
		args2___,
		xseq_, 
		args3___
		]]:= qe`plus[args1, args2, args3, qe`times[2,xseq]];


Scan[Function[lista,
With[{quantumsum=lista[[1]],mathematicasum=lista[[2]]},Module[{unprotected},
	unprotected=Unprotect[quantumsum];

(* The efficient work of the rules of quantumsum requires
the attribute HoldAllComplete*)
	SetAttributes[quantumsum, HoldAllComplete];
	
	Options[quantumsum]=Options[mathematicasum];
	
	(* hermitian conjugate of sum *)
	quantumsum /: 
		HoldPattern[qe`hermitianConjugate[quantumsum[arg_,{index_Symbol,limits___},opts:OptionsPattern[]]]]:=
			quantumsum[qe`hermitianConjugate[arg],{index,limits},opts];
	
	quantumsum /: 
		HoldPattern[qe`expectation[quantumsum[arg_,{index_Symbol,limits___},opts:OptionsPattern[]]]]:=
			quantumsum[qe`expectation[arg],{index,limits},opts];	
	
	(* qsum of qplus becomes qplus of qsum if the qsum is not an infinite sum *)
	(* Notice that Infinity evaluates to DirectedInfinity[1], so we use Unevaluated[Infinity]*)
	HoldPattern[quantumsum[qe`plus[argsplus__],{lims___},opts:OptionsPattern[]]]/;
			And[FreeQ[HoldComplete[lims],Infinity],
				FreeQ[HoldComplete[lims],Unevaluated[Infinity]],
				FreeQ[HoldComplete[lims],DirectedInfinity],
				FreeQ[HoldComplete[lims],DirectedInfinity[___]],
				FreeQ[HoldComplete[lims],ComplexInfinity],
				FreeQ[HoldComplete[lims],Unevaluated[ComplexInfinity]]]:=
			Module[{auxsum,expresion},
				SetAttributes[auxsum,HoldAllComplete];
				qi`setUndefined[auxsum];
				HoldPattern[auxsum[argu_]]:=quantumsum[argu,{lims},opts]; (* limits must not be evaluated *)
				expresion=Map[auxsum,HoldComplete[argsplus]]; (* argsplus must not be evaluated *)
				expresion=Apply[List,expresion]; (* force auxsum to be replaced by quantumsum *)
				ClearAll[auxsum];
				Apply[qe`plus,expresion] ];
		
(* Mathematica version 10.4 syntax coloring shows 
the "index" variable below, inside "Block", in red color, 
suggesting a scoping issue or problem in the following code.
Depiste the red coloring of Mathematica 10.4 in the following code,
it is correct, it behaves the way I want to, see this link:
http://mathematica.stackexchange.com/questions/115722
*)
(* A scalar pushes-out nonscalars that do not depend on the index out of the sum,
   to obtain a quantumsum with scalara argument that will be later
   transformed to a standard Mathematica Sum.
   Assuming scalars were automatically moved by previous definitions
   to the begining (left) of qe`times and
   are automatically transformed to a single scalar (usually with head Times)
 *)
	HoldPattern[
			quantumsum[qe`times[insidescalar_?qe`scalarQ,
							rightnonscalars:((_?qe`nonscalarQ)..)],
					{index_Symbol,limits___},opts:OptionsPattern[]]]:=
	Block[{index},
		qe`times[quantumsum[insidescalar,{index,limits},opts],
				qe`times[rightnonscalars]] /;
		Apply[And,Map[FreeQ[#,index]&,{rightnonscalars}]]
	];
	
(* inside nonscalar that depends on the dummy index of the sum
	pulls-in other constant nonscalars into the sum,
	with the hope that nonscalar with nonscalar will
	give you a scalar (bra times ket to braket), 
	if that happens then the quantumsum will be transformed to a 
	standard Mathematica Sum that later will be evaluated.
	The nonscalar that is going to be pulled into the
	sum must Not contain (FreeQ) a symbol identical
	to the index of the sum, THIS IS IMPORTANT WHEN
	one sum will pull-in other sum to generate nested sums,
	it will not happend until they have differente dummy indexes,
	thanks to another rule below
 *)
	quantumsum /: HoldPattern[
			qe`times[leftargs___,
					leftnonscalar_?qe`nonscalarQ,
					quantumsum[insidenonscalar_?qe`nonscalarQ,{index_Symbol,limits___},opts:OptionsPattern[]],
					rightargs___]]:=
	Block[{index},
		qe`times[leftargs,
			quantumsum[qe`times[leftnonscalar,insidenonscalar],{index,limits},opts],
			rightargs] /;
		And[FreeQ[leftnonscalar,index],
			Not[FreeQ[insidenonscalar,index]]]
	];

	quantumsum /: HoldPattern[
			qe`times[leftargs___,
					quantumsum[insidenonscalar_?qe`nonscalarQ,{index_Symbol,limits___},opts:OptionsPattern[]],
					rightnonscalar_?qe`nonscalarQ,
					rightargs___]]:=
	Block[{index},
		qe`times[leftargs,
			quantumsum[qe`times[insidenonscalar,rightnonscalar],{index,limits},opts],
			rightargs] /;
		And[FreeQ[rightnonscalar,index],
			Not[FreeQ[insidenonscalar,index]]]
	];

(* The argument of the quantumsum is evaluated within the scope 
of Block, which localizes the index, making it a dummy variable,
independent of a possible global variable with the same name.
This will make qe`times[bra,ket] to be transformed to braket,
which is a scalar, and later the quantumsum will be transformed to a Sum 
THE EFFICIENT USE OF THIS RULE REQUIRES THE ATTRIBUTE HOLDALLCOMPLETE FOR quantumsum,
SEE THE SECOND ANSWER IN http://mathematica.stackexchange.com/questions/115722
 *)
	HoldPattern[quantumsum[arg_, {index_Symbol, limits___},opts:OptionsPattern[]]]:=
	Block[{index},
		Module[{evaluatedarg},
			qi`logPrint["quantumsum"," blocked evaluation of argument ",HoldComplete[arg]];
			evaluatedarg = arg;
			With[{replaceevaluated = evaluatedarg},
	            quantumsum[replaceevaluated, {index, limits},opts] /;
	            HoldComplete[replaceevaluated] =!= HoldComplete[arg]] 
	] ];

(* The product of two nonscalar qsums that have the same dummy index
changes the dummy index of both, so they have different dummy indexes,
and later, because of previous definitions,
one of them will pull-in the other one to generate nested qsums,
maybe making the argument of those nested qsums become an scalar,
and later transforming the qsums to normal Mathematica sums
 *)
	quantumsum /: HoldPattern[
			qe`times[leftargs___,
				quantumsum[nonscalar1_?qe`nonscalarQ,{sameindex_Symbol,limits1___},opts1:OptionsPattern[]],
				quantumsum[nonscalar2_?qe`nonscalarQ,{sameindex_Symbol,limits2___},opts2:OptionsPattern[]],
				rightargs___]]:=
	Block[{sameindex},
	   qi`logPrint["quantumsum"," two sums with same index ",HoldComplete[sameindex]];
		qe`times[leftargs,
			With[{newindex=Unique[ToString[sameindex]]},
				ReleaseHold[Hold[quantumsum[nonscalar1,{newindex,limits1},opts1]]/.
					sameindex:>newindex]],
			With[{newindex=Unique[ToString[sameindex]]},
				ReleaseHold[Hold[quantumsum[nonscalar2,{newindex,limits2},opts2]]/.
					sameindex:>newindex]],
			rightargs]
	];

(* Finally, after all the previuos rules have been applied,
if the argument of a quantumsum became a scalar and 
it does not contain any of the qe quantum equivalents of numeric functions
(see condition after /; at the end of the Block localizing the index),
then the quantumsum is transformed into 
a standard Mathematica sum for evaluation *)
	HoldPattern[quantumsum[scalar_?qe`scalarQ,{index_Symbol,limits___},opts:OptionsPattern[]]]:=
		Block[{index},
			qi`logPrint["quantumsum"," quantumsum to mathematicasum ",HoldComplete[scalar]];
			mathematicasum[scalar,{index,limits},opts]/;
				Apply[And, Map[FreeQ[HoldComplete[scalar],#]&,
						qi`$numfunprerules[[All,2]]]]];

(* Use Sum rules inside QSum when we have a multiplication of
 a scalar function of the summation index and a quantum object 
 that can be also a function of the summation index.
 The trick is to create a Sum with the quantum object replaced by 
 a totally inert scalar function of the index, so that Mathematica 
 applies its rich rules for working with KroneckerDelta, UnitStep,
 Piecewise, etc. to the original scalar function without "disturbing"
 the inert scalar function (EXCEPT ON ITS ARGUMENT, 
 THIS CHANGE IS WHAT I WANT IN THE quantum object),
 Then this inert scalar function, with its new argument,
  will be replaced back to the 
 original quantum object. Notice that qobject is transformed
 into a function in order to make the "back-replacement"
 for example:
 quantumsum[(Delta[j,10]+Delta[j,20])*Ket1[j+7]*Ket2[j+9]]
 Sum[(Delta[j,10]+Delta[j,20])*inert[j]]
 inert[10]+inert[20]
 inert[10]+inert[20] /. inert\[Rule]((Ket1[#1+7]*Ket2[#1+9])&)
 ((Ket1[#1+7]*Ket2[#1+9])&)[10]+((Ket1[#1+7]*Ket2[#1+9])&)[20]
 Ket1[17]*Ket2[19]+Ket1[27]*Ket2[29] 
  *)
	HoldPattern[quantumsum[qe`times[scalar_?qe`scalarQ,qobjects__?qe`nonscalarQ],
						{index_Symbol, limits___},opts:OptionsPattern[]]/;
					Apply[And, Map[FreeQ[scalar,#]&, qi`$numfunprerules[[All,2]]]]]:=
	Block[{index},
	Module[{inertfunction,evaluatedsum,resultsum},
		With[{replaceevaluated=evaluatedsum, 
			rule=(inertfunction->((qe`times[qobjects] &)/.index->Slot[1]))},
			qi`logPrint["quantumsum"," back-replacement rule ",rule];
			resultsum=ReleaseHold[
				( HoldComplete[replaceevaluated]/.rule )//.qi`$preRules];
			qi`logPrint["quantumsum"," result ",resultsum];
			resultsum] /; 
		And[Not[FreeQ[HoldComplete[scalar],index]],
			Not[FreeQ[HoldComplete[qe`times[qobjects]],index]],
			(evaluatedsum=mathematicasum[Times[scalar,inertfunction[index]],{index, limits},opts];
			With[{replaceevaluated=evaluatedsum},
					qi`logPrint["quantumsum"," inert function trick evaluated ",HoldComplete[replaceevaluated]];
					qi`logPrint["quantumsum"," inert function trick q-object  ",HoldComplete[qe`times[qobjects]]];
					HoldComplete[replaceevaluated]=!=
						HoldComplete[mathematicasum[Times[scalar,inertfunction[index]],{index, limits},opts]]])]]];
	
	Protect[Evaluate[unprotected]] ] ] ],
{{qe`sum, Sum},
 {qe`nsum, NSum},
 {qe`integrate, Integrate},
 {qe`nintegrate, NIntegrate}
} ];


Scan[Function[lista,
With[{quantumprod=lista[[1]],mathematicaprod=lista[[2]]},
Module[{unprotected},
	unprotected=Unprotect[quantumprod];
	
	(* The efficient work of the rules of quantumprod requires
	the attribute HoldAllComplete*)
	SetAttributes[quantumprod, HoldAllComplete];

	Options[quantumprod]=Options[mathematicaprod];

	(* quantumprod of scalar becomes normal product *)
	HoldPattern[quantumprod[scalar_?qe`scalarQ,{index_Symbol,limits___},opts:OptionsPattern[]]]:=
		Block[{index},
			qi`logPrint["quantumprod"," quantumprod to mathematicaprod ",HoldComplete[scalar]];
			mathematicaprod[scalar,{index,limits},opts]/;
				Apply[And, Map[FreeQ[HoldComplete[scalar],#]&,
						qi`$numfunprerules[[All,2]]]]];
	
	(* Blocked evaluation making a dummy (local) index *)		
	HoldPattern[quantumprod[arg_, {index_Symbol, limits___},opts:OptionsPattern[]]]:=
	Block[{index},
		Module[{evaluatedarg},
			qi`logPrint["quantumsum"," blocked evaluation of argument ",HoldComplete[arg]];
			evaluatedarg = arg;
			With[{replaceevaluated = evaluatedarg},
	            quantumprod[replaceevaluated, {index, limits},opts] /;
	            HoldComplete[replaceevaluated] =!= HoldComplete[arg]] 
	]];
	
	Protect[Evaluate[unprotected]]
	] ] ],
{{qe`product, Product},
 {qe`nproduct, NProduct}
} ];

(* iterator with symbol and one or two integer limits AND WITHOUT OPTIONS *)
HoldPattern[qe`product[nonscalar_?qe`nonscalarQ,{index_Symbol,start_Integer:1,end_Integer}]]:=
		Block[{index},
			qi`logPrint["quantumprod"," quantumprod to qetimes ",HoldComplete[nonscalar]];
			Apply[qe`times,Table[nonscalar,{index,start,end}]]];
			
(* iterator with symbol, two integer limits, and an integer increment AND WITHOUT OPTIONS 
It cannot be included in the one before because of the optional argument start_Integer:1,
otherwise Mathematica might confuse
 {index, start, end} with {index, optional:1, end, increment} *)
HoldPattern[qe`product[nonscalar_?qe`nonscalarQ,
				{index_Symbol,start_Integer,end_Integer,inc_Integer}]]:=
		Block[{index},
			qi`logPrint["quantumprod"," quantumprod to qetimes (2) ",HoldComplete[nonscalar]];
			Apply[qe`times,Table[nonscalar,{index,start,end,inc}]]];


(* qe`qmutator of two arguments is transformed to commutator *)
HoldPattern[qe`qmutator[a_,b_]]:=qe`qmutator[a,b,1];


qe`qmutator /: 
	HoldPattern[qe`hermitianConjugate[qe`qmutator[a_,b_,q_]]]:=
		qe`qmutator[qe`hermitianConjugate[b],qe`hermitianConjugate[a],qe`conjugate[q]];


(* A non-ordered q-mutator is by default
transformed into the corresponding expression depending on the ordered one *)
HoldPattern[ qe`qmutator[b_,a_,q_]/; Not[qi`normalOrderQ[b,a]] ]:=
	qe`plus[qe`times[-q,qe`qmutator[a,b,q]],qe`times[1-q^2,b,a]];


(* If the end-user made an assignment to the non-ordered q-mutator
then the previous rule will not apply, and the non-ordered one will evaluate
to a result that is FreeQ of the HoldPattern[ordered one]. 
In that case the ordered one is transformed into the corresponding
expression depending on the non-ordered one, therefore using the end-user 
assignment
*)
qi`qmutatorOutsideFlag=True;
HoldPattern[ qe`qmutator[a_,b_,q_]/; 
	And[qi`qmutatorOutsideFlag,
		qi`normalOrderQ[a,b],
		Block[{qi`qmutatorOutsideFlag=False},
			FreeQ[qe`qmutator[b,a,q],HoldPattern[qe`qmutator[a,b,q]]]] ] ]:=
		Block[{qi`qmutatorOutsideFlag=False},
			qe`plus[qe`times[-q,qe`qmutator[b,a,q]],qe`times[1-q^2,a,b]]];


HoldPattern[qe`qmutator[qe`times[sca_?qe`scalarQ,c__],b_,q_]/;
				And[FreeQ[{sca,c,b},qe`ket],FreeQ[{sca,c,b},qe`bra]]]:=
	qe`times[sca,qe`qmutator[qe`times[c],b,q]];


HoldPattern[qe`qmutator[a_,qe`times[sca_?qe`scalarQ,d__],q_]/;
				And[FreeQ[{a,sca,d},qe`ket],FreeQ[{a,sca,d},qe`bra]]]:=
	qe`times[sca,qe`qmutator[a,qe`times[d],q]];


HoldPattern[qe`qmutator[a_, sca_?qe`scalarQ, q_]]:=
	qe`times[sca*(1-q),a];


HoldPattern[qe`qmutator[sca_?qe`scalarQ, b_, q_]]:=
	qe`times[sca*(1-q),b];


HoldPattern[qe`qmutator[a_, a_, q_]]:=
	qe`times[1-q,qe`power[a,2]];


(* Remeber that qe`symmetric is NOT a "real operation", 
because this transformation is NOT a function one-to-one
of the input expression and output expression.
It is more like a short-notation, similar to the normal-order notation,
therefore its rules are not real algebraic rules, it is more like
constructing from the original expression another symmetrized expression
that later could be used to represent PART of the original expression
  *)
HoldPattern[qe`symmetric[qe`plus[args__]]]:=
	qe`plus@@Map[qe`symmetric,{args}];
	
HoldPattern[qe`symmetric[qe`times[args__]]]:=
	With[{sorted=Sort[{args},qi`normalOrderQ]},
		qe`symmetric[qe`times@@sorted] /;
		{args}=!=sorted
	];
	
HoldPattern[qe`symmetric[qe`times[sca_?qe`scalarQ,args__]]]:=
	qe`times[sca,qe`symmetric[qe`times[args]]];
		
HoldPattern[qe`symmetric[anyother:Except[(_qe`times|_qe`plus)]]]:=anyother;  


(* Finishing this package *)


Protect[Evaluate[unprotected]];
End[];


qi`message=qi`hello[];


EndPackage[];

Column[{qi`message, qi`sessionAliases[]}]
