(* ::Package:: *)

BeginPackage["qm`qmSlots`",{"qm`qmBasic`","qm`qmTypesetting`","Global`"}];


(* 
   Copyright 2016
   Jose Luis Gomez-Munoz and Francisco Delgado
   ITESM-CEM, 
   Departamento de F\[IAcute]sica y Matem\[AAcute]ticas (Ciencias B\[AAcute]sicas)
   Carretera Lago de Guadalupe Km. 3.5, 
   Atizap\[AAcute]n de Zaragoza, Estado de M\[EAcute]xico, 
   C.P. 52926
   Mexico
   jose.luis.gomez@itesm.mx
   http://homepage.cem.itesm.mx/jose.luis.gomez/
*)


Unprotect[qi`$slotNames];
ClearAll[qi`$slotNames];
qi`$slotNames=HoldComplete[
			(* information lists *)
			qi`$undefined, qi`$operators, qi`$kets, qi`$bras,
			qi`$scalarHeads,
			(* programmer commands *)
			qi`setUndefined, qi`setOperator, qi`setKet, qi`setBra,
			qi`slotlabels, qi`slotnumbers,
			qi`verifySlotLabels, qi`obtainSlotNumbers,
			qi`subsinhead,qi`subsinheadQ
];
Apply[Unprotect,qi`$slotNames];
Apply[ClearAll,qi`$slotNames];
Scan[Function[#::usage=ToString[#]<>" is a symbol of the Quantum Mechanics packages"],
	qi`$slotNames];
(* Next command does NOT protect the names (symbols) inside the list qi`$slotNames;
   it DOES protect the name of the list itself *)
SetAttributes[qi`$slotNames,{Protected,ReadProtected}];
qi`$slotNames::usage="qi`$slotNames is an internal symbol of the Quantum Mechanics packages";


(*These list sould be modified only with the qmSet* commands *)
qi`$undefined={}; qi`$operators={}; qi`$kets={}; qi`$bras={};
(*System symbols h that do not have attribute NumericFuntion 
but the expression h[args] is scalar*)
qi`$scalarHeads=
	{Rational,Complex,Root,RootSum,
	DirectedInfinity,DiracDelta,HeavisideTheta,
	Sum,NSum,Integrate,NIntegrate,Product,NProduct};
(*These lists are protected, they should be modified only with qmSet* commands *)
Protect[qi`$undefined, qi`$operators, qi`$kets, qi`$bras, qi`$scalarHeads];


(* The attribute must be HoldAll instead of HoldAllComplete because
must rules for slotlabels are defined as upvalues of the argument,
and upvalues do Not work if the attribute is HoldAllComplete *)
SetAttributes[qi`slotlabels,{HoldAll,Listable}];
SetAttributes[qi`slotnumbers,{HoldAll,Listable}];


Options[qi`setOperator]={ "SlotLabels"->
								{{1,Missing["Undefined"]}} };
Options[qi`setKet]={ "SlotLabels"->
								{{1,Missing["Undefined"]}} };
Options[qi`setBra]={ "SlotLabels"->
								{{1,Missing["Undefined"]}} };


Begin["`Private`"];


HoldPattern[qi`slotnumbers[args___]]:=qi`obtainSlotNumbers[qi`slotlabels[args]];


(* Zero arguments, maybe from Sequence[] *)
HoldPattern[qi`slotlabels[]]:=Missing["Undefined"];


(* Only one argument for qi`slotlabels *)
HoldPattern[qi`slotlabels[one_,more__]]:=
	(Message[qi`slotlabels::argx,qi`slotlabels,1+Length[Unevaluated[{more}]]];Abort[]);


(* It is necessary to specify the behaviour in a CompoundExpression because
the attribute HoldAll of qi`slotlabels prevents the CompoundExpression from
evaluating to its last argument *)
HoldPattern[qi`slotlabels[CompoundExpression[args___,last_]]]:=
	qi`verifySlotLabels[qi`slotlabels[last]];


(* System commands like Map[args], Apply[args], etc
that are not NumericFunctions neither Constants,
have Missing[Undefined] slots, because we do not know a priori 
the type of result *)
HoldPattern[qi`slotlabels[sym_Symbol[args___]]/;
	And[FreeQ[qi`$scalarHeads,sym],
		Context[sym]==="System`",
		FreeQ[Attributes[sym],NumericFunction],
		FreeQ[Attributes[sym],Constant]
	]]:=Missing["Undefined"];


(* If no particular rules for finding slot are asigned to the complete expression, 
then it will have the same slotlabels as the head of the expression. 
Important for subscripted operators with parameters *)
HoldPattern[qi`slotlabels[thehead_[args___]]]:= 
	qi`slotlabels[thehead];


(* Mathematica default commands like Plus, Times, assume everything is 
a complex number, except lists. In that spirit, qi`slotlabels assumes everything
to be a scalar (complex number), except lists (it is Listable), and also except those
symbols and structures that have explicit qi`slotlabels[].
Notice that any scalar is also an operator in common notation used by papers,
it represents the scalar times the identity operator *)
HoldPattern[qi`slotlabels[anyother:Except[_List]]]:={{},{}};


qi`setUndefined[sym_Symbol]:=
Module[{unprotected},

	unprotected=Unprotect[sym,qi`slotlabels,qi`$undefined];

(* remember the rule for sym implies the rules for
 sym[args] and even sym[args1][args2], 
because of a previous general rule of qi`slotlabels *)
	sym /: qi`slotlabels[HoldPattern[sym]] := 
		Missing["Undefined"];
	sym::qmslotsmsg=
		ToString[HoldForm[sym]]<>" and "<>ToString[HoldForm[sym]]<>
		"[args] cannot be used for the INTERNAL representation of Kets, Bras, Operators nor Scalars";
	SetAttributes[sym,ReadProtected];
	qi`$undefined=Union[qi`$undefined,{HoldComplete[sym]}];

	Protect[Evaluate[unprotected]];
	sym::qmslotsmsg
];


qi`setOperator[sym_Symbol,opts:OptionsPattern[]]:=
Module[{unprotected},

	unprotected=Unprotect[sym,qe`subscript,qi`slotlabels,qi`$operators];

	Switch[OptionValue["SlotLabels"],
		"Subscripted",(
			qe`subscript /: qi`slotlabels[HoldPattern[qe`subscript[sym[arg1_],sequ__]]] := 
						With[{labels=Sort[Map[{#,arg1}&,{sequ}]]},qi`verifySlotLabels[{labels,labels}]];
			sym /: qi`slotlabels[HoldPattern[sym[arg1_]]] := 
						qi`verifySlotLabels[{{{1,arg1}},{{1,arg1}}}];
			sym /: qi`slotlabels[HoldPattern[sym]] := 
						Missing["Undefined"];
			sym::qmslotsmsg=ToString[sym]<>" represents an operator with slot labels "<>
							"identified with subscripts, it must be used in expressions wrapped by qm[ ]";
		),
		"SecondArgument",(
			sym /: qi`slotlabels[HoldPattern[sym[arg1_,arg2_List,argmore___]]] := 
						With[{labels=Sort[Map[{#,arg1}&,arg2]]},qi`verifySlotLabels[{labels,labels}]];
(* remember the rule for sym implies the rules for
 sym[args] and even sym[args1][args2], 
because of a previous general rule of qi`slotlabels *)
			sym /: qi`slotlabels[HoldPattern[sym]] := 
						Missing["Undefined"];
			sym::qmslotsmsg=ToString[sym]<>" represents an operator with slot labels "<>
							"identified with subscripts, it must be used in expressions wrapped by qm[ ]";
		),
		_,(
(* remember the rule for sym implies the rules for
 sym[args] and even sym[args1][args2], 
because of a previous general rule of qi`slotlabels *)
			sym /: qi`slotlabels[HoldPattern[sym]] := 
						qi`verifySlotLabels[{OptionValue["SlotLabels"],OptionValue["SlotLabels"]}];
			sym::qmslotsmsg=ToString[sym]<>" represents an operator with slots "<>
					ToString[qi`verifySlotLabels[{OptionValue["SlotLabels"],OptionValue["SlotLabels"]}]]<>
					", it must be used in expressions wrapped by qm[ ]";
		)];
	SetAttributes[sym,ReadProtected];
	qi`$operators=Union[qi`$operators,{HoldComplete[sym]}];

	Protect[Evaluate[unprotected]];
	sym::qmslotsmsg
];


SetAttributes[qi`notsubsinheadQ,{HoldAll}];
qi`notsubsinheadQ[args___]:=Not[qi`subsinheadQ[args]];

SetAttributes[qi`subsinheadQ,{HoldAll}];
qi`subsinheadQ[qe`subscript[___]]:=True;
qi`subsinheadQ[thehead_[args___]]:= qi`subsinheadQ[thehead];
qi`subsinheadQ[anyonther_]:=False;

SetAttributes[qi`subsinhead,{HoldAll}];
qi`subsinhead[qe`subscript[first_,args__]]:=Map[{#,first}&,{args}];
qi`subsinhead[thehead_[args___]]:= qi`subsinhead[thehead];
qi`subsinhead[anyonther_]:={};

qi`setKet[sym_Symbol,opts:OptionsPattern[]]:=
Module[{unprotected},

	unprotected=Unprotect[sym,qi`slotlabels,qi`$kets];

	Switch[OptionValue["SlotLabels"],
		"SubscriptedArguments",( 
			sym /: qi`slotlabels[HoldPattern[sym[args:((_?qi`subsinheadQ)..)]]] := 
					qi`verifySlotLabels[{Sort[Flatten[Map[qi`subsinhead[#]&,{args}],1]],{}}];
			sym /: qi`slotlabels[HoldPattern[sym[args:((_?qi`notsubsinheadQ)..)]]] := 
					qi`verifySlotLabels[{Sort[MapIndexed[{#2[[1]],#1}&,{args}]],{}}];
(* remember the rule for sym implies the rules for
 sym[args] and even sym[args1][args2], 
because of a previous general rule of qi`slotlabels *)
			sym /: qi`slotlabels[HoldPattern[sym]] :=
					Missing["Undefined"];
			sym::qmslotsmsg=ToString[sym]<>" represents a ket with slot labels "<>
							"identified with subscripted arguments,"<>
							" it must be used in expressions wrapped by qm[ ]";
		),
		_,(
(* remember the rule for sym implies the rules for
 sym[args] and even sym[args1][args2], 
because of a previous general rule of qi`slotlabels *)
			sym /: qi`slotlabels[HoldPattern[sym]] := 
						qi`verifySlotLabels[{OptionValue["SlotLabels"],{}}];
			sym::qmslotsmsg=ToString[sym]<>" represents a ket with slots "<>
							ToString[qi`verifySlotLabels[{OptionValue["SlotLabels"],{}}]]<>
							", it must be used in expressions wrapped by qm[ ]";
		)];

	SetAttributes[sym,ReadProtected];
	qi`$kets=Union[qi`$kets,{HoldComplete[sym]}];

	Protect[Evaluate[unprotected]];
	sym::qmslotsmsg
];

qi`setBra[sym_Symbol,opts:OptionsPattern[]]:=
Module[{unprotected},

	unprotected=Unprotect[sym,qi`slotlabels,qi`$bras];

	Switch[OptionValue["SlotLabels"],
		"SubscriptedArguments",( 
			sym /: qi`slotlabels[HoldPattern[sym[args:((_?qi`subsinheadQ)..)]]] :=
					qi`verifySlotLabels[{{},Sort[Flatten[Map[qi`subsinhead[#]&,{args}],1]]}];
			sym /: qi`slotlabels[HoldPattern[sym[args:((_?qi`notsubsinheadQ)..)]]] :=
					qi`verifySlotLabels[{{},Sort[MapIndexed[{#2[[1]],#1}&,{args}]]}];
(* remember the rule for sym implies the rules for
 sym[args] and even sym[args1][args2], 
because of a previous general rule of qi`slotlabels *)
			sym /: qi`slotlabels[HoldPattern[sym]] :=
					Missing["Undefined"];
			sym::qmslotsmsg=ToString[sym]<>" represents a bra with slot labels "<>
							"identified with subscripts, it must be used in expressions wrapped by qm[ ]";
		),
		_,(
(* remember the rule for sym implies the rules for
 sym[args] and even sym[args1][args2], 
because of a previous general rule of qi`slotlabels *)
			sym /: qi`slotlabels[HoldPattern[sym]] := 
						qi`verifySlotLabels[{{},OptionValue["SlotLabels"]}];
			sym::qmslotsmsg=ToString[sym]<>" represents a bra with slots "<>
							ToString[qi`verifySlotLabels[{{},OptionValue["SlotLabels"]}]]<>
							", it must be used in expressions wrapped by qm[ ]";
		)];

	SetAttributes[sym,ReadProtected];
	qi`$bras=Union[qi`$bras,{HoldComplete[sym]}];

	Protect[Evaluate[unprotected]];
	sym::qmslotsmsg
];



qi`verifySlotLabels[]:=Missing["Undefined"]; (* zero arguments *)

qi`verifySlotLabels[arg1_,argmore__]:=Missing["Undefined"]; (* two or more arguments *)

qi`verifySlotLabels[propList_]:=
Module[{verifiedslots, sortedketlabels, sortedbralabels},
	verifiedslots=propList;
	
	Which[
		verifiedslots===Missing["Nonscalar"], 
			verifiedslots,
		MatchQ[verifiedslots,{ {{_Integer,_}...},{{_Integer,_}...} }],
			sortedketlabels=Union[ verifiedslots[[1]],SameTest->(First[#1]==First[#2]&) ];
			sortedbralabels=Union[ verifiedslots[[2]],SameTest->(First[#1]==First[#2]&) ];
			If[TrueQ[verifiedslots==={sortedketlabels,sortedbralabels}],
				verifiedslots,
				Missing["Undefined"]],
		True, (* invalid, it is Undefined*)
			Missing["Undefined"]
	]
];


qi`obtainSlotNumbers[]:=Missing["Undefined"]; (* zero arguments *)

qi`obtainSlotNumbers[arg1_,argmore__]:=Missing["Undefined"]; (* two or more arguments *)

qi`obtainSlotNumbers[slabels_]:=
Module[{verifiedslots, sortedketlabels, sortedbralabels},
	verifiedslots=qi`verifySlotLabels[slabels];
	Which[
		MatchQ[verifiedslots,{ {{_Integer,_}..},{{_Integer,_}..} }],
			sortedketlabels=Union[ verifiedslots[[1]],SameTest->(First[#1]==First[#2]&) ];
			sortedbralabels=Union[ verifiedslots[[2]],SameTest->(First[#1]==First[#2]&) ];
			{Transpose[sortedketlabels][[1]],Transpose[sortedbralabels][[1]]},
		MatchQ[verifiedslots,{ {{_Integer,_}..},{} }],
			sortedketlabels=Union[ verifiedslots[[1]],SameTest->(First[#1]==First[#2]&) ];
			sortedbralabels=verifiedslots[[2]];
			{Transpose[sortedketlabels][[1]],sortedbralabels},
		MatchQ[verifiedslots,{ {},{{_Integer,_}..} }],
			sortedketlabels=verifiedslots[[1]];
			sortedbralabels=Union[ verifiedslots[[2]],SameTest->(First[#1]==First[#2]&) ];
			{sortedketlabels,Transpose[sortedbralabels][[1]]},
		MatchQ[verifiedslots,{ {},{} }],
				verifiedslots,
		True, (* invalid, it is Undefined *)
			Missing["Undefined"]
	]
];


(* Finishing this package *)


General::qmundefined=
	"cannot determine either the state space of `1` or if it is a ket, bra, operator or scalar";
qi`setUndefined::usage="qi`setUndefined[symbol] specifies that symbol"<>
" cannot represent a ket, bra, operator not scalar in expressions wrapped by qm[ ]";
qi`setOperator::usage="qi`setOperator[symbol] specifies that symbol is a"<>
" noncommutative operator that must be used in expressions wrapped by qm[ ]";
qi`setKet::usage="qi`setKet[symbol] specifies that symbol is a"<>
" Ket that must be used in expressions wrapped by qm[ ]";
qi`setBra::usage="qi`setBra[symbol] specifies that symbol is a"<>
" Bra that must be used in expressions wrapped by qm[ ]";
qi`$undefined::usage="qi`$undefined is the list of symbols that represent Operators";
qi`$operators::usage="qi`$operators is the list of symbols that represent Operators";
qi`$kets::usage="qi`$kets is the list of symbols that represent Kets";
qi`$bras::usage="qi`$bras is the list of symbols that represent Bras";


qi`setCommands[
	qi`setUndefined, qi`setOperator, qi`setKet, qi`setBra
];

qi`setUnlockedCommands[
	qi`slotlabels, qi`slotnumbers,
	qi`verifySlotLabels, qi`obtainSlotNumbers,
	qi`subsinhead,qi`subsinheadQ
];


End[];


qi`message=qi`hello[];


EndPackage[];

qi`message
