(* ::Package:: *)

BeginPackage["qm`qmSlotsExpressions`",
	{"qm`qmBasic`","qm`qmTypesetting`",
	"qm`qmSlots`","qm`qmSlotsPrimitives`","Global`"}];


(* 
   Copyright 2016
   Jose Luis Gomez-Munoz and Francisco Delgado
   ITESM-CEM, 
   Departamento de F\[IAcute]sica y Matem\[AAcute]ticas (Ciencias B\[AAcute]sicas)
   Carretera Lago de Guadalupe Km. 3.5, 
   Atizap\[AAcute]n de Zaragoza, Estado de M\[EAcute]xico, 
   C.P. 52926
   Mexico
   jose.luis.gomez@itesm.mx
   http://homepage.cem.itesm.mx/jose.luis.gomez/
*)


Begin["`Private`"];


(* By default, numeric functions and 
their qe quantum equivalents 
are scalars if all their arguments are scalars.
If any of the arguments is nonscalar,
then numeric functions are undefined (because they should Abort[], see qmOperations.m),
however their qe quantum equivalents are nonscalars
*)
Module[{unprotected},
	unprotected=
		Union[Unprotect[qi`slotlabels],
			Unprotect[Evaluate[qi`$numfunprerules[[All,2]]]]];
(* remember the rule for sym implies the rules for
 sym[], sym[args] and even sym[args1][args2], 
because of a previous general rule of qi`slotlabels *)
	Do[
		With[{qefun=qi`$numfunprerules[[nf,2]],fun=qi`$numfunprerules[[nf,1]]},
			qefun /: qi`slotlabels[HoldPattern[qefun]]:=Missing["Nonscalar"];
			qefun /: qi`slotlabels[HoldPattern[qefun[args:((_?qe`scalarQ)..)]]]:={{},{}};
			qi`slotlabels[HoldPattern[fun]]:=Missing["Undefined"];
			qi`slotlabels[HoldPattern[fun[args:((_?qe`scalarQ)..)]]]:={{},{}} ],
	{nf,1,Length[qi`$numfunprerules]}];

	qi`slotlabels[HoldPattern[Sum]]:=Missing["Undefined"];
	qi`slotlabels[HoldPattern[Sum[arg1_?qe`scalarQ,args__]]]:={{},{}};
	qi`slotlabels[HoldPattern[NSum]]:=Missing["Undefined"];
	qi`slotlabels[HoldPattern[NSum[arg1_?qe`scalarQ,args__]]]:={{},{}};
	qi`slotlabels[HoldPattern[Integrate]]:=Missing["Undefined"];
	qi`slotlabels[HoldPattern[Integrate[arg1_?qe`scalarQ,args__]]]:={{},{}};
	qi`slotlabels[HoldPattern[NIntegrate]]:=Missing["Undefined"];
	qi`slotlabels[HoldPattern[NIntegrate[arg1_?qe`scalarQ,args__]]]:={{},{}};
	qi`slotlabels[HoldPattern[Product]]:=Missing["Undefined"];
	qi`slotlabels[HoldPattern[Product[arg1_?qe`scalarQ,args__]]]:={{},{}};
	qi`slotlabels[HoldPattern[NProduct]]:=Missing["Undefined"];
	qi`slotlabels[HoldPattern[NProduct[arg1_?qe`scalarQ,args__]]]:={{},{}};
	
	Protect[Evaluate[unprotected]] ];


Module[{unprotected},
	unprotected=Unprotect[qe`brkt];
(* remember the rule for sym implies the rules for
 sym[], sym[args] and even sym[args1][args2], 
because of a previous general rule of qi`slotlabels *)
	qe`brkt /: qi`slotlabels[HoldPattern[qe`brkt]]:=
		Missing["Undefined"];
(* next rule is important for unbalanced bra-kets that are not real brakets*)
	qe`brkt /: qi`slotlabels[HoldPattern[qe`brkt[{braargs___},{ketargs___}]]]:=
		qi`slotlabels[qe`times[qe`bra[braargs],qe`ket[ketargs]]];
	Protect[Evaluate[unprotected]] ];


Module[{unprotected},
	unprotected=Unprotect[qe`times];
(* remember the rule for sym implies the rules for
 sym[], sym[args] and even sym[args1][args2], 
because of a previous general rule of qi`slotlabels *)
	qe`times /: qi`slotlabels[HoldPattern[qe`times]]:=
		Missing["Undefined"];
	qe`times /: qi`slotlabels[HoldPattern[qe`times[args__]]]:=
	Module[{labelslist,labelssofar,j,
			ketlabelsfirst,bralabelsfirst,ketlabelssecond,bralabelssecond,
			newbralabelsfirst,newketlabelssecond},
		qi`logPrint["slot labels for qe`times of: ",HoldComplete[{args}]];
		labelslist=Map[qi`slotlabels,Unevaluated[{args}]];
		qi`logPrint["qe`times slolt labels list: ",labelslist];
		labelslist=Map[qi`verifySlotLabels,labelslist];
		qi`logPrint["qe`times verifyed slolt labels list: ",labelslist];
		Which[
		TrueQ[MemberQ[labelslist,Missing["Undefined"],{1}]], (*Ok, MemberQ only searches at level 1*)
			Missing["Undefined"], (* if one argument is undefined, also the command is undefined *)
		TrueQ[MemberQ[labelslist,Missing["Nonscalar"],{1}]], (*Ok, MemberQ only searches at level 1 *)
			Missing["Nonscalar"], (* if one argument is nonscalar-without-slots, also is the command *)			
		True, (* otherwise review each argument to determine the final slot labels *)
			labelssofar=labelslist[[1]];
			Do[
				{ketlabelsfirst,bralabelsfirst}=labelssofar;
				{ketlabelssecond,bralabelssecond}=labelslist[[j]];
				newbralabelsfirst=
					Complement[bralabelsfirst,ketlabelssecond,SameTest->(First[#1]==First[#2]&)];
				newketlabelssecond=
					Complement[ketlabelssecond,bralabelsfirst,SameTest->(First[#1]==First[#2]&)];
				labelssofar={Sort[Join[ketlabelsfirst,newketlabelssecond]], (*Not Union in order to...*)
							Sort[Join[newbralabelsfirst,bralabelssecond]]}, (*...identify some ketket errors*)
				{j,2,Length[labelslist]}]
			qi`logPrint["qe`times final slotlabels: ",labelssofar];
			labelssofar=qi`verifySlotLabels[labelssofar];
			qi`logPrint["qe`times verified final slotlabels: ",labelssofar];
			labelssofar (* final result *)
		] ];
	Protect[Evaluate[unprotected]] ];


Module[{unprotected},
	unprotected=Unprotect[qe`plus];
(* remember the rule for sym implies the rules for
 sym[], sym[args] and even sym[args1][args2], 
because of a previous general rule of qi`slotlabels *)
	qe`plus /: qi`slotlabels[HoldPattern[qe`plus]]:=
		Missing["Undefined"];
	qe`plus /: qi`slotlabels[HoldPattern[qe`plus[args__]]]:=
	Module[{labelslist,labelssofar,j,k,
			ketlabelsfirst,bralabelsfirst,ketlabelssecond,bralabelssecond,
			newbralabels,newketlabels},
		qi`logPrint["slot labels for qe`plus of: ",HoldComplete[{args}]];
		labelslist=Map[qi`slotlabels,Unevaluated[{args}]];
		qi`logPrint["qe`plus slolt labels list: ",labelslist];
		labelslist=Map[qi`verifySlotLabels,labelslist];
		qi`logPrint["qe`plus verifyed slolt labels list: ",labelslist];
		Which[
		TrueQ[MemberQ[labelslist,Missing["Undefined"],{1}]], (*Ok, MemberQ only searches at level 1*)
			Missing["Undefined"], (* if one argument is undefined, also the command is undefined *)
		TrueQ[MemberQ[labelslist,Missing["Nonscalar"],{1}]], (*Ok, MemberQ only searches at level 1 *)
			Missing["Nonscalar"], (* if one argument is nonscalar-without-slots, also is the command *)			
		True, (* otherwise review each argument to determine the final slot labels *)
			labelssofar=labelslist[[1]];
			Do[
				{ketlabelsfirst,bralabelsfirst}=labelssofar;
				{ketlabelssecond,bralabelssecond}=labelslist[[j]];
				Which[
					And[labelssofar==={{},{}},
							ketlabelssecond[[All,1]]===bralabelssecond[[All,1]]], (* scalar+operator *)
						labelssofar={ketlabelssecond,bralabelssecond},
					And[ketlabelsfirst[[All,1]]===bralabelsfirst[[All,1]],  (* operator+scalar *)
							labelslist[[j]]==={{},{}}], 
						labelssofar,
					qi`obtainSlotNumbers[labelssofar]===qi`obtainSlotNumbers[labelslist[[j]]],
						(newketlabels=ketlabelsfirst;
						Do[If[newketlabels[[k]]=!=ketlabelssecond[[k]],
								newketlabels[[k]]={newketlabels[[k,1]],Missing["Undefined"]}],
							{k,1,Length[newketlabels]}];
						newbralabels=bralabelsfirst;
						Do[If[newbralabels[[k]]=!=bralabelssecond[[k]],
								newbralabels[[k]]={newbralabels[[k,1]],Missing["Undefined"]}],
							{k,1,Length[bralabelsfirst]}];
						labelssofar={newketlabels,newbralabels}),
					True,
						labelssofar=Missing["Undefined"]; Break[] (*Exit Do, it is undefined*)
				],
				{j,2,Length[labelslist]}]
			qi`logPrint["qe`plus final slotlabels: ",labelssofar];
			labelssofar=qi`verifySlotLabels[labelssofar];
			qi`logPrint["qe`plus verified final slotlabels: ",labelssofar];
			labelssofar (* final result *)
		] ];
	Protect[Evaluate[unprotected]] ];


Module[{unprotected},
	unprotected=Unprotect[qe`power];
(* remember the rule for sym implies the rules for
 sym[], sym[args] and even sym[args1][args2], 
because of a previous general rule of qi`slotlabels *)
	qe`power /: qi`slotlabels[HoldPattern[qe`power]]:=
		Missing["Undefined"];
	qe`power /: qi`slotlabels[HoldPattern[qe`power[base_,exponent_]]]:=
	Module[{labelslist,labelssofar,labelsbase,labelssuper},
		qi`logPrint["slot labels for qe`power of: ",HoldComplete[{base,exponent}]];
		labelslist=Map[qi`slotlabels,Unevaluated[{base,exponent}]];
		qi`logPrint["qe`power slolt labels list: ",labelslist];
		labelslist=Map[qi`verifySlotLabels,labelslist];
		qi`logPrint["qe`power verifyed slolt labels list: ",labelslist];
		Which[
		TrueQ[MemberQ[labelslist,Missing["Undefined"],{1}]], (*Ok, MemberQ only searches at level 1*)
			Missing["Undefined"], (* if one argument is undefined, also the command is undefined *)
		TrueQ[MemberQ[labelslist,Missing["Nonscalar"],{1}]], (*Ok, MemberQ only searches at level 1 *)
			Missing["Nonscalar"], (* if one argument is nonscalar-without-slots, also is the command *)			
		True, (* otherwise review each argument to determine the final slot labels *)
			labelsbase=labelslist[[1]];
			labelssuper=labelslist[[2]];
			Which[
					labelsbase===labelssuper==={{},{}}, (* scalar^scalar *)
						labelssofar={{},{}},
					And[labelsbase==={{},{}}, labelssuper[[1]]!={}, labelssuper[[2]]!={}, 
					labelssuper[[1,All,1]]===labelssuper[[2,All,1]]], (* scalar^operator *)
						labelssofar=labelssuper,
					And[labelssuper==={{},{}}, labelsbase[[1]]!={}, labelsbase[[2]]!={}, 
					labelsbase[[1,All,1]]===labelsbase[[2,All,1]]], (* operator^scalar *)
						labelssofar=labelsbase,
					True,
						labelssofar=Missing["Undefined"] (*It is undefined*)
				];
			qi`logPrint["qe`power final slotlabels: ",labelssofar];
			labelssofar=qi`verifySlotLabels[labelssofar];
			qi`logPrint["qe`power verified final slotlabels: ",labelssofar];
			labelssofar (* final result *)
		] ];
	Protect[Evaluate[unprotected]] ]


Module[{unprotected},
	unprotected=Unprotect[qe`qmutator];
(* remember the rule for sym implies the rules for
 sym[], sym[args] and even sym[args1][args2], 
because of a previous general rule of qi`slotlabels *)
	qe`qmutator /: qi`slotlabels[HoldPattern[qe`qmutator]]:=
		Missing["Undefined"];
	qe`qmutator /: qi`slotlabels[HoldPattern[qe`qmutator[ope1_,ope2_,q_]]]:=
	Module[{labelslist,labelssofar,labelsope1,labelsope2},
		qi`logPrint["slot labels for qe`qmutator of: ",HoldComplete[{ope1,ope2}]];
		labelslist=Map[qi`slotlabels,Unevaluated[{ope1,ope2}]];
		qi`logPrint["qe`qmutator slolt labels list: ",labelslist];
		labelslist=Map[qi`verifySlotLabels,labelslist];
		qi`logPrint["qe`qmutator verifyed slolt labels list: ",labelslist];
		Which[
		TrueQ[MemberQ[labelslist,Missing["Undefined"],{1}]], (*Ok, MemberQ only searches at level 1*)
			Missing["Undefined"], (* if one argument is undefined, also the command is undefined *)
		TrueQ[MemberQ[labelslist,Missing["Nonscalar"],{1}]], (*Ok, MemberQ only searches at level 1 *)
			Missing["Nonscalar"], (* if one argument is nonscalar-without-slots, also is the command *)			
		True, (* otherwise review each argument to determine the final slot labels *)
			labelsope1=labelslist[[1]];
			labelsope2=labelslist[[2]];
			Which[
					labelsope1===labelsope2==={{},{}}, (* [scalar,scalar] *)
						labelssofar={{},{}},
					And[labelsope1==={{},{}}, labelsope2[[1]]!={}, labelsope2[[2]]!={}, 
					labelsope2[[1,All,1]]===labelsope2[[2,All,1]]], (* [scalar,operator] *)
						labelssofar=labelsope2,
					And[labelsope2==={{},{}}, labelsope1[[1]]!={}, labelsope1[[2]]!={}, 
					labelsope1[[1,All,1]]===labelsope1[[2,All,1]]], (* [operator,scalar] *)
						labelssofar=labelsope1,
					And[labelsope1[[1]]!={}, labelsope1[[2]]!={}, 
					labelsope2[[1]]!={}, labelsope2[[2]]!={},
					labelsope1[[1,All,1]]===labelsope1[[2,All,1]],
					labelsope2[[1,All,1]]===labelsope2[[2,All,1]]], (* [operator,operator] *)
						labelssofar=qi`slotlabels[qe`plus[qe`times[ope1,ope2],qe`times[ope2,ope1]]],
					True,
						labelssofar=Missing["Undefined"] (*It is undefined*)
				];
			qi`logPrint["qe`qmutator final slotlabels: ",labelssofar];
			labelssofar=qi`verifySlotLabels[labelssofar];
			qi`logPrint["qe`qmutator verified final slotlabels: ",labelssofar];
			labelssofar (* final result *)
		] ];
	Protect[Evaluate[unprotected]] ];


Module[{unprotected},
	unprotected=Unprotect[qe`hermitianConjugate];
(* remember the rule for sym implies the rules for
 sym[], sym[args] and even sym[args1][args2], 
because of a previous general rule of qi`slotlabels *)
	qe`hermitianConjugate /: qi`slotlabels[HoldPattern[qe`hermitianConjugate]]:=
		Missing["Undefined"];
	qe`hermitianConjugate /: qi`slotlabels[HoldPattern[qe`hermitianConjugate[arg_]]]:=
	Module[{labelslist,labelssofar},
		qi`logPrint["slot labels for qe`hermitianConjugate of: ",HoldComplete[{arg}]];
		labelslist=Map[qi`slotlabels,Unevaluated[{arg}]];
		qi`logPrint["qe`hermitianConjugate slolt labels list: ",labelslist];
		labelslist=Map[qi`verifySlotLabels,labelslist];
		qi`logPrint["qe`hermitianConjugate verifyed slolt labels list: ",labelslist];
		Which[
		TrueQ[MemberQ[labelslist,Missing["Undefined"],{1}]], (*Ok, MemberQ only searches at level 1*)
			Missing["Undefined"], (* if one argument is undefined, also the command is undefined *)
		TrueQ[MemberQ[labelslist,Missing["Nonscalar"],{1}]], (*Ok, MemberQ only searches at level 1 *)
			Missing["Nonscalar"], (* if one argument is nonscalar-without-slots, also is the command *)			
		True, (* otherwise review each argument to determine the final slot labels *)
			labelssofar={labelslist[[1,2]],labelslist[[1,1]]};
			qi`logPrint["qe`hermitianConjugate final slotlabels: ",labelssofar];
			labelssofar=qi`verifySlotLabels[labelssofar];
			qi`logPrint["qe`hermitianConjugate verified final slotlabels: ",labelssofar];
			labelssofar (* final result *)
		] ];
	Protect[Evaluate[unprotected]] ];


Module[{unprotected},
	unprotected=Unprotect[qe`expectation];
(* remember the rule for sym implies the rules for
 sym[], sym[args] and even sym[args1][args2], 
because of a previous general rule of qi`slotlabels *)
	qe`expectation /: qi`slotlabels[HoldPattern[qe`expectation]]:=
		Missing["Undefined"];
	qe`expectation /: qi`slotlabels[HoldPattern[qe`expectation[ope_]]]:=
	Module[{labelslist,labelssofar,labelsope},
		qi`logPrint["slot labels for qe`expectation of: ",HoldComplete[{ope}]];
		labelslist=Map[qi`slotlabels,Unevaluated[{ope}]];
		qi`logPrint["qe`expectation slolt labels list: ",labelslist];
		labelslist=Map[qi`verifySlotLabels,labelslist];
		qi`logPrint["qe`expectation verifyed slolt labels list: ",labelslist];
		Which[
		TrueQ[MemberQ[labelslist,Missing["Undefined"],{1}]], (*Ok, MemberQ only searches at level 1*)
			Missing["Undefined"], (* if one argument is undefined, also the command is undefined *)
		TrueQ[MemberQ[labelslist,Missing["Nonscalar"],{1}]], (*Ok, MemberQ only searches at level 1 *)
			Missing["Nonscalar"], (* if one argument is nonscalar-without-slots, also is the command *)			
		True, (* otherwise review each argument to determine the final slot labels *)
			labelsope=labelslist[[1]];
			Which[
					labelsope==={{},{}}, (* <scalar> *)
						labelssofar={{},{}},
					And[labelsope[[1]]!={}, labelsope[[2]]!={}, 
					labelsope[[1,All,1]]===labelsope[[2,All,1]]], (* <operator> *)
						labelssofar={{},{}},
					True,
						labelssofar=Missing["Undefined"] (*It is undefined*)
				];
			qi`logPrint["qe`expectation final slotlabels: ",labelssofar];
			labelssofar=qi`verifySlotLabels[labelssofar];
			qi`logPrint["qe`expectation verified final slotlabels: ",labelssofar];
			labelssofar (* final result *)
		] ];
	Protect[Evaluate[unprotected]] ];


Module[{unprotected},
	unprotected=Unprotect[qe`conjugate];
(* remember the rule for sym implies the rules for
 sym[], sym[args] and even sym[args1][args2], 
because of a previous general rule of qi`slotlabels *)
	qe`conjugate /: qi`slotlabels[HoldPattern[qe`conjugate]]:=
		Missing["Undefined"];
	qe`conjugate /: qi`slotlabels[HoldPattern[qe`conjugate[scalar_]]]:=
	Module[{labelslist,labelssofar,labelsarg},
		qi`logPrint["slot labels for qe`conjugate of: ",HoldComplete[{scalar}]];
		labelslist=Map[qi`slotlabels,Unevaluated[{scalar}]];
		qi`logPrint["qe`conjugate slolt labels list: ",labelslist];
		labelslist=Map[qi`verifySlotLabels,labelslist];
		qi`logPrint["qe`conjugate verifyed slolt labels list: ",labelslist];
		Which[
		TrueQ[MemberQ[labelslist,Missing["Undefined"],{1}]], (*Ok, MemberQ only searches at level 1*)
			Missing["Undefined"], (* if one argument is undefined, also the command is undefined *)
		TrueQ[MemberQ[labelslist,Missing["Nonscalar"],{1}]], (*Ok, MemberQ only searches at level 1 *)
			Missing["Nonscalar"], (* if one argument is nonscalar-without-slots, also is the command *)			
		True, (* otherwise review each argument to determine the final slot labels *)
			labelsarg=labelslist[[1]];
			Which[
					labelsarg==={{},{}}, (* <scalar> *)
						labelssofar={{},{}},
					True,
						labelssofar=Missing["Undefined"] (*It is undefined*)
				];
			qi`logPrint["qe`conjugate final slotlabels: ",labelssofar];
			labelssofar=qi`verifySlotLabels[labelssofar];
			qi`logPrint["qe`conjugate verified final slotlabels: ",labelssofar];
			labelssofar (* final result *)
		] ];
	Protect[Evaluate[unprotected]] ];


Module[{unprotected},
	unprotected=Unprotect[qe`symmetric];
(* remember the rule for sym implies the rules for
 sym[], sym[args] and even sym[args1][args2], 
because of a previous general rule of qi`slotlabels *)
	qe`symmetric /: qi`slotlabels[HoldPattern[qe`symmetric]]:=
		Missing["Undefined"];
	qe`symmetric /: qi`slotlabels[HoldPattern[qe`symmetric[args__]]]:=
	Module[{labelslist,labelssofar,labelsope},
		qi`logPrint["slot labels for qe`symmetric of: ",HoldComplete[{args}]];
		labelslist=Map[qi`slotlabels,Unevaluated[{args}]];
		qi`logPrint["qe`symmetric slolt labels list: ",labelslist];
		labelslist=Map[qi`verifySlotLabels,labelslist];
		qi`logPrint["qe`symmetric verifyed slolt labels list: ",labelslist];
		labelssofar=labelslist[[1]];
		qi`logPrint["qe`symmetric final slotlabels: ",labelssofar];
		labelssofar=qi`verifySlotLabels[labelssofar];
		qi`logPrint["qe`symmetric verified final slotlabels: ",labelssofar];
		labelssofar (* final result *)
	];
	Protect[Evaluate[unprotected]] ];


Module[{unprotected},
	unprotected=Unprotect[qe`integrate];
(* remember the rule for sym implies the rules for
 sym[], sym[args] and even sym[args1][args2], 
because of a previous general rule of qi`slotlabels *)
	qe`integrate /: qi`slotlabels[HoldPattern[qe`integrate]]:=
		Missing["Undefined"];
	qe`integrate /: qi`slotlabels[HoldPattern[qe`integrate[args__]]]:=
	Module[{labelslist,labelssofar,labelsope},
		qi`logPrint["slot labels for qe`integrate of: ",HoldComplete[{args}]];
		labelslist=Map[qi`slotlabels,Unevaluated[{args}]];
		qi`logPrint["qe`integrate slolt labels list: ",labelslist];
		labelslist=Map[qi`verifySlotLabels,labelslist];
		qi`logPrint["qe`integrate verifyed slolt labels list: ",labelslist];
		labelssofar=labelslist[[1]];
		qi`logPrint["qe`integrate final slotlabels: ",labelssofar];
		labelssofar=qi`verifySlotLabels[labelssofar];
		qi`logPrint["qe`integrate verified final slotlabels: ",labelssofar];
		labelssofar (* final result *)
	];
	Protect[Evaluate[unprotected]] ];


Module[{unprotected},
	unprotected=Unprotect[qe`nintegrate];
(* remember the rule for sym implies the rules for
 sym[], sym[args] and even sym[args1][args2], 
because of a previous general rule of qi`slotlabels *)
	qe`nintegrate /: qi`slotlabels[HoldPattern[qe`nintegrate]]:=
		Missing["Undefined"];
	qe`nintegrate /: qi`slotlabels[HoldPattern[qe`nintegrate[args__]]]:=
	Module[{labelslist,labelssofar,labelsope},
		qi`logPrint["slot labels for qe`nintegrate of: ",HoldComplete[{args}]];
		labelslist=Map[qi`slotlabels,Unevaluated[{args}]];
		qi`logPrint["qe`nintegrate slolt labels list: ",labelslist];
		labelslist=Map[qi`verifySlotLabels,labelslist];
		qi`logPrint["qe`nintegrate verifyed slolt labels list: ",labelslist];
		labelssofar=labelslist[[1]];
		qi`logPrint["qe`nintegrate final slotlabels: ",labelssofar];
		labelssofar=qi`verifySlotLabels[labelssofar];
		qi`logPrint["qe`nintegrate verified final slotlabels: ",labelssofar];
		labelssofar (* final result *)
	];
	Protect[Evaluate[unprotected]] ];


Module[{unprotected},
	unprotected=Unprotect[qe`sum];
(* remember the rule for sym implies the rules for
 sym[], sym[args] and even sym[args1][args2], 
because of a previous general rule of qi`slotlabels *)
	qe`sum /: qi`slotlabels[HoldPattern[qe`sum]]:=
		Missing["Undefined"];
	qe`sum /: qi`slotlabels[HoldPattern[qe`sum[args__]]]:=
	Module[{labelslist,labelssofar,labelsope},
		qi`logPrint["slot labels for qe`sum of: ",HoldComplete[{args}]];
		labelslist=Map[qi`slotlabels,Unevaluated[{args}]];
		qi`logPrint["qe`sum slolt labels list: ",labelslist];
		labelslist=Map[qi`verifySlotLabels,labelslist];
		qi`logPrint["qe`sum verifyed slolt labels list: ",labelslist];
		labelssofar=labelslist[[1]];
		qi`logPrint["qe`sum final slotlabels: ",labelssofar];
		labelssofar=qi`verifySlotLabels[labelssofar];
		qi`logPrint["qe`sum verified final slotlabels: ",labelssofar];
		labelssofar (* final result *)
	];
	Protect[Evaluate[unprotected]] ];


Module[{unprotected},
	unprotected=Unprotect[qe`nsum];
(* remember the rule for sym implies the rules for
 sym[], sym[args] and even sym[args1][args2], 
because of a previous general rule of qi`slotlabels *)
	qe`nsum /: qi`slotlabels[HoldPattern[qe`nsum]]:=
		Missing["Undefined"];
	qe`nsum /: qi`slotlabels[HoldPattern[qe`nsum[args__]]]:=
	Module[{labelslist,labelssofar,labelsope},
		qi`logPrint["slot labels for qe`nsum of: ",HoldComplete[{args}]];
		labelslist=Map[qi`slotlabels,Unevaluated[{args}]];
		qi`logPrint["qe`nsum slolt labels list: ",labelslist];
		labelslist=Map[qi`verifySlotLabels,labelslist];
		qi`logPrint["qe`nsum verifyed slolt labels list: ",labelslist];
		labelssofar=labelslist[[1]];
		qi`logPrint["qe`nsum final slotlabels: ",labelssofar];
		labelssofar=qi`verifySlotLabels[labelssofar];
		qi`logPrint["qe`nsum verified final slotlabels: ",labelssofar];
		labelssofar (* final result *)
	];
	Protect[Evaluate[unprotected]] ];


Module[{unprotected},
	unprotected=Unprotect[qe`product];
(* remember the rule for sym implies the rules for
 sym[], sym[args] and even sym[args1][args2], 
because of a previous general rule of qi`slotlabels *)
	qe`product /: qi`slotlabels[HoldPattern[qe`product]]:=
		Missing["Undefined"];
	Protect[Evaluate[unprotected]] ];


Module[{unprotected},
	unprotected=Unprotect[qe`nproduct];
(* remember the rule for sym implies the rules for
 sym[], sym[args] and even sym[args1][args2], 
because of a previous general rule of qi`slotlabels *)
	qe`nproduct /: qi`slotlabels[HoldPattern[qe`nproduct]]:=
		Missing["Undefined"];
	Protect[Evaluate[unprotected]] ];


Module[{unprotected},
	unprotected=Unprotect[qe`superscript];
(* remember the rule for sym implies the rules for
 sym[], sym[args] and even sym[args1][args2], 
because of a previous general rule of qi`slotlabels *)
	qe`superscript /: qi`slotlabels[HoldPattern[qe`superscript]]:=
		Missing["Undefined"];
	Protect[Evaluate[unprotected]] ];


Module[{unprotected},
	unprotected=Unprotect[qe`subscript];
(* remember the rule for sym implies the rules for
 sym[], sym[args] and even sym[args1][args2], 
because of a previous general rule of qi`slotlabels *)
	qe`subscript /: qi`slotlabels[HoldPattern[qe`subscript]]:=
		Missing["Undefined"];
	Protect[Evaluate[unprotected]] ];


Module[{unprotected},
	unprotected=Unprotect[qe`superMinus];
(* remember the rule for sym implies the rules for
 sym[], sym[args] and even sym[args1][args2], 
because of a previous general rule of qi`slotlabels *)
	qe`superMinus /: qi`slotlabels[HoldPattern[qe`superMinus]]:=
		Missing["Undefined"];
	Protect[Evaluate[unprotected]] ];


Module[{unprotected},
	unprotected=Unprotect[qe`subMinus];
(* remember the rule for sym implies the rules for
 sym[], sym[args] and even sym[args1][args2], 
because of a previous general rule of qi`slotlabels *)
	qe`subMinus /: qi`slotlabels[HoldPattern[qe`subMinus]]:=
		Missing["Undefined"];
	Protect[Evaluate[unprotected]] ];


Module[{unprotected},
	unprotected=Unprotect[qe`superPlus];
(* remember the rule for sym implies the rules for
 sym[], sym[args] and even sym[args1][args2], 
because of a previous general rule of qi`slotlabels *)
	qe`superPlus /: qi`slotlabels[HoldPattern[qe`superPlus]]:=
		Missing["Undefined"];
	Protect[Evaluate[unprotected]] ];


Module[{unprotected},
	unprotected=Unprotect[qe`subPlus];
(* remember the rule for sym implies the rules for
 sym[], sym[args] and even sym[args1][args2], 
because of a previous general rule of qi`slotlabels *)
	qe`subPlus /: qi`slotlabels[HoldPattern[qe`subPlus]]:=
		Missing["Undefined"];
	Protect[Evaluate[unprotected]] ];


(* qm[ ] must repace the qmCommands so far by the correponding qe`commands
before asking for slot labels. Therefore, if slot labels of qmCommands get asked,
it is very likely the command is not wrapped by qm[ ], 
or it did not get replaced because of an error,
therefore their slots are undefined.
It also handles nested qm[] "injected" inside qm[] by the
use of % *)
Map[qi`setUndefined,Map[Symbol,Names["qm`Dirac`qm*"]]];


End[];


qi`message=qi`hello[];


EndPackage[];

Column[{qi`message, qi`sessionAliases[]}]
