(* ::Package:: *)

BeginPackage["qm`qmSlotsPrimitives`",
	{"qm`qmBasic`","qm`qmTypesetting`","qm`qmSlots`","Global`"}];


(* 
   Copyright 2016
   Jose Luis Gomez-Munoz and Francisco Delgado
   ITESM-CEM, 
   Departamento de F\[IAcute]sica y Matem\[AAcute]ticas (Ciencias B\[AAcute]sicas)
   Carretera Lago de Guadalupe Km. 3.5, 
   Atizap\[AAcute]n de Zaragoza, Estado de M\[EAcute]xico, 
   C.P. 52926
   Mexico
   jose.luis.gomez@itesm.mx
   http://homepage.cem.itesm.mx/jose.luis.gomez/
*)


Unprotect[qi`$slotsPrimitivesNames];
ClearAll[qi`$slotsPrimitivesNames];
qi`$slotsPrimitivesNames=HoldComplete[
			(* end-user qmcommands *)
			qm`Dirac`qmScalarQ,
			qm`Dirac`qmNonScalarQ,
			qm`Dirac`qmUnknownQ,
			(* internal qe`commands *)
			qe`scalarQ,
			qe`nonscalarQ,
			qe`unknownQ,
			(* programmer commands *)
			qi`$maxScalarQDefinitions,
			qi`storedScalarQDefinitions,
			qi`autoResetScalarQ,
			qi`resetScalarQ,
			qi`$maxUnknownQDefinitions,
			qi`storedUnknownQDefinitions,
			qi`autoResetUnknownQ,
			qi`resetUnknownQ

];
Apply[Unprotect,qi`$slotsPrimitivesNames];
Apply[ClearAll,qi`$slotsPrimitivesNames];
Scan[Function[#::usage=ToString[#]<>" is a symbol of the Quantum Mechanics packages"],
	qi`$slotsPrimitivesNames];
(* Next command does NOT protect the names (symbols) inside the list qi`$PrimitivesPropertiesettingNames;
   it DOES protect the name of the list itself *)
SetAttributes[qi`$slotsPrimitivesNames,{Protected,ReadProtected}];
qi`$slotsPrimitivesNames::usage="qi`$slotsPrimitivesNames is an internal symbol of the Quantum Mechanics packages";


SetAttributes[qe`scalarQ,HoldAllComplete];
SetAttributes[qe`nonscalarQ,HoldAllComplete];
SetAttributes[qe`unknownQ,HoldAllComplete];


Begin["`Private`"];


(* Primitives that represent Kets where 
either all arguments are subscripted (the subscripts indicate the slots) 
or all arguments are nonsubscripted (the order of the arguments indicates the slots) *)
Map[qi`setKet[#,"SlotLabels"->"SubscriptedArguments"]&,
	Map[Symbol,Names["qe`ket*"]]];


(* Primitives that represent Bras where 
either all arguments are subscripted (the subscripts indicate the slots) 
or all arguments are nonsubscripted (the order of the arguments indicates the slots) *)
Map[qi`setBra[#,"SlotLabels"->"SubscriptedArguments"]&,
	Map[Symbol,Names["qe`bra*"]]];


(* Primitives that represent subscripted Operators  
(the subscripts indicate the slots)  *)
Map[qi`setOperator[#,"SlotLabels"->"Subscripted"]&,
	Map[Symbol,Names["qe`ope*"]]];


qi`setPreRules[
	HoldPattern[qm`Dirac`qmScalarQ]->qe`scalarQ,
	HoldPattern[qm`Dirac`qmNonScalarQ]->qe`nonscalarQ,
	HoldPattern[qm`Dirac`qmUnknownQ]->qe`unknownQ
];

qi`setPostRules[
	HoldPattern[qe`scalarQ]->qm`Dirac`qmScalarQ,
	HoldPattern[qe`nonscalarQ]->qm`Dirac`qmNonScalarQ,
	HoldPattern[qe`unknownQ]->qm`Dirac`qmUnknownQ
];

qi`setUndefined[qe`scalarQ]; (* boolean *)
qi`setUndefined[qe`nonscalarQ]; 
qi`setUndefined[qe`unknownQ]; 




qi`$maxScalarQDefinitions=100;

qi`autoResetScalarQ[]:=
Module[{},
	If[qi`storedScalarQDefinitions>qi`$maxScalarQDefinitions,
		qi`resetScalarQ[]
	];
	qi`storedScalarQDefinitions++
];

qi`resetScalarQ[]:=
Module[{},
	Clear[qe`scalarQ];
	qi`storedScalarQDefinitions=0;
	(* ALL qe`scalarQ definitions MUST be declared here, inside qi`resetScalarQ,
		so that they can be restored after the Clear[qe`scalarQ] that clears the
		memoization *)
	HoldPattern[qe`scalarQ[qe`ket[___]]]:=False;
	HoldPattern[qe`scalarQ[qe`bra[___]]]:=False;
	(* Definition with MEMOIZATION and automatic RESETTING of memoization *)
	HoldPattern[qe`scalarQ[anyother_]]:=
		HoldPattern[ Verbatim[qe`scalarQ[anyother]] ]=
			(qi`autoResetScalarQ[]; TrueQ[qi`slotlabels[anyother]=={{},{}}]);
];

(* First-time definition of qe`scalarQ *)
qi`resetScalarQ[];


qi`$maxUnknownQDefinitions=100;

qi`autoResetUnknownQ[]:=
Module[{},
	If[qi`storedUnknownQDefinitions>qi`$maxUnknownQDefinitions,
		qi`resetUnknownQ[]
	];
	qi`storedUnknownQDefinitions++
];

qi`resetUnknownQ[]:=
Module[{},
	Clear[qe`unknownQ];
	qi`storedUnknownQDefinitions=0;
	(* ALL qe`unknownQ definitions MUST be declared here, inside qi`resetUnknownQ,
		so that they can be restored after the Clear[qe`unknownQ] that clears the
		memoization *)
	HoldPattern[qe`unknownQ[qe`ket[___]]]:=False;
	HoldPattern[qe`unknownQ[qe`bra[___]]]:=False;
	(* Definition with MEMOIZATION and automatic RESETTING of memoization *)
	HoldPattern[qe`unknownQ[anyother_]]:=
		HoldPattern[ Verbatim[qe`unknownQ[anyother]] ]=
			(qi`autoResetUnknownQ[]; TrueQ[qi`slotlabels[anyother]==Missing["Undefined"]]);
];

(* First-time definition of qe`unknownQ *)
qi`resetUnknownQ[];


qe`nonscalarQ[any_]:=
	And[Not[qe`scalarQ[any]],Not[qe`unknownQ[any]]];


(* Finishing this package *)


qi`setCommands[
			qi`$maxUnknownQDefinitions,
			qi`storedUnknownQDefinitions,
			qi`autoResetUnknownQ,
			qi`resetUnknownQ,
			qi`$maxScalarQDefinitions,
			qi`storedScalarQDefinitions,
			qi`autoResetScalarQ,
			qi`resetScalarQ
];

qi`setUnlockedCommands[
	qe`scalarQ, qe`nonscalarQ, qe`unknownQ
];

Unprotect[qe`scalarQ]; (* IMPORTANT for memoization *)
Unprotect[qe`unknownQ]; (* IMPORTANT for memoization *)

qi`setClearAllSymbols[
	qm`Dirac`qmScalarQ, qm`Dirac`qmNonScalarQ, qm`Dirac`qmUnknownQ
];


qe`scalarQ::usage="qe`scalarQ[expr] gives True if expr is a scalar, and False otherwise";
qe`nonscalarQ::usage="qe`nonscalarQ[expr] gives True if expr is a nonscalar, and False otherwise";
qe`unknownQ::usage="qe`unknownQ[expr] gives True if it cannot be determined whether expr is scalar or nonscalar";
(* Error Messages*)
General::qmnonscalar="Scalar expected instead of nonscalar argument `1` ";


End[];


qi`message=qi`hello[];


EndPackage[];

Column[{qi`message, qi`sessionAliases[]}]
