(* ::Package:: *)

BeginPackage["qm`qmTypesetting`",{"qm`qmBasic`", "Global`"}];
(* We need to import the Global` context, otherwise we get two shadowing 
Dagger symbols if the user has assigned a value to Dagger before loading the 
package *)


(* 
   Copyright 2017
   Jose Luis Gomez-Munoz and Francisco Delgado
   ITESM-CEM, 
   Departamento de F\[IAcute]sica y Matem\[AAcute]ticas
   Carretera Lago de Guadalupe Km. 3.5, 
   Atizap\[AAcute]n de Zaragoza, Estado de M\[EAcute]xico, 
   C.P. 52926
   Mexico
   jose.luis.gomez@itesm.mx
   http://homepage.cem.itesm.mx/jose.luis.gomez/
*)


Unprotect[qi`$typesettingNames];
ClearAll[qi`$typesettingNames];
qi`$typesettingNames=HoldComplete[
		(* end-user qmcommands *)
			qm`Dirac`qmQmutator, qm`Dirac`qmOverHat, 
			qm`Dirac`qmKet, qm`Dirac`qmBra, qm`Dirac`qmBrkt,
		(* qe`commands *)
			qe`traditionalform,
			qe`qmutator,
			qe`symmetric, 
			qe`ket, qe`bra, qe`brkt,
			qe`subscript,
			qe`superscript,
			qe`subPlus,
			qe`subMinus,
			qe`superPlus,
			qe`superMinus,
			qe`opeHat,
			qe`expectation,
			qe`hermitianConjugate,
			qe`integrate, qe`sum, qe`product, 
			qe`nintegrate, qe`nsum, qe`nproduct,
		(* programmer commands, append to data lists: *)
			qi`setOpeHatHermitian,
			qi`setOpeHatSubscriptHermitian,
			qi`setOpeHatSuper,
			qi`setOpeHatSubscriptSuper,
			qi`setOpeHatSubPlusMinusSuper,
			qi`dagger,
			qi`typesettingHoldComplete,
			qi`typesettingTimes,
		(* programmer data lists *)
			qi`$numfunstr,
			qi`$numfunprerules,
			qi`$numfunposrules,
			qi`$opeHatHermitian,
			qi`$opeHatSubscriptHermitian,
			qi`$opeHatSuper,
			qi`$opeHatSubscriptSuper,
			qi`$opeHatSubPlusMinusSuper,
		(* Template boxes *)
			qi`braTemplateBox,
			qi`brktTemplateBox,
			qi`ketTemplateBox,
			qi`overHatTemplateBox,
			qi`qmutatorTemplateBox,
			qi`qmutatorTraditionalTemplateBox
];
Apply[Unprotect,qi`$typesettingNames];
Apply[ClearAll,qi`$typesettingNames];
Scan[Function[#::usage=ToString[#]<>" is a symbol of the Quantum Mechanics packages"],
	qi`$typesettingNames];

qi`$numfunstr=Select[Names["System`*"],MemberQ[Attributes[#],NumericFunction]&];
qi`$numfunprerules=Map[Symbol[#]->Symbol["qe`"<>ToLowerCase[#]]&,qi`$numfunstr];
qi`$numfunposrules=Map[Symbol["qe`"<>ToLowerCase[#]]-> Symbol[#]&,qi`$numfunstr];
Apply[Unprotect,qi`$numfunprerules[[All,2]]];
Apply[ClearAll,qi`$numfunprerules[[All,2]]];
Scan[Function[#::usage=ToString[#]<>" is a noncommutative algebraic function of the Quantum Mechanics packages"],
	qi`$numfunprerules[[All,2]]];
qi`$typesettingNames=Join[qi`$typesettingNames,Apply[HoldComplete,qi`$numfunprerules[[All,2]]]];

(* Next command does NOT protect the names (symbols) inside the list qi`$typesettingNames;
   it DOES protect the name of the list itself *)
SetAttributes[qi`$typesettingNames,{Protected,ReadProtected}];
qi`$typesettingNames::usage="qi`$typesettingNames is an internal symbol of the Quantum Mechanics packages";


(* Next list should be modified only with the qi`set* commands *)
(* Data lists *)
qi`$opeHatHermitian={};
qi`$opeHatSubscriptHermitian={};
qi`$opeHatSuper={};
qi`$opeHatSubscriptSuper={};
qi`$opeHatSubPlusMinusSuper={};

(*These lists are protected, they should be modified only with qi`set* commands *)
Protect[
(* Data lists *)
		qi`$numfunstr,
		qi`$numfunprerules,
		qi`$numfunposrules,
		qi`$opeHatHermitian,
		qi`$opeHatSubscriptHermitian,
		qi`$opeHatSuper,
		qi`$opeHatSubscriptSuper,
		qi`$opeHatSubPlusMinusSuper];
(* Programmer Data lists are read-protected. Information lists are not*)
SetAttributes[{
		qi`$numfunstr,
		qi`$numfunprerules,
		qi`$numfunposrules,
		qi`$opeHatHermitian,
		qi`$opeHatSubscriptHermitian,
		qi`$opeHatSuper,
		qi`$opeHatSubscriptSuper,
		qi`$opeHatSubPlusMinusSuper
	},
	ReadProtected
];


SetAttributes[{
		qi`setOpeHatHermitian,
		qi`setOpeHatSubscriptHermitian,
		qi`setOpeHatSuper,
		qi`setOpeHatSubscriptSuper,
		qi`setOpeHatSubPlusMinusSuper
	},
	HoldAllComplete
];
SetAttributes[{
	qe`subscript,
	qe`superscript
	},
	NHoldRest
];


(* Let SuperDagger, SuperStar OverHat and AngleBracket be modified symbols, 
so they are mentioned at the welcome message, and also 
clearallsymbols, so we have the message that they 
must be used inside qm[] *)
qi`setImpossibleSymbols[Global`\[Dagger], Global`\[ScriptS]];
qi`setModifiedSymbols[SuperDagger, SuperStar, OverHat, AngleBracket]; 
qi`setClearAllSymbols[
	qm`Dirac`qmQmutator, qm`Dirac`qmOverHat, 
	qm`Dirac`qmKet, qm`Dirac`qmBra, qm`Dirac`qmBrkt,
	SuperDagger, SuperStar, OverHat, AngleBracket
];


Begin["`Private`"];


qi`setOpeHatHermitian[new_]:=
Module[{},
	Unprotect[qi`$opeHatHermitian];
	AppendTo[qi`$opeHatHermitian,HoldComplete[new]];
	Protect[qi`$opeHatHermitian];
]; 


qi`setOpeHatSubscriptHermitian[new_]:=
Module[{},
	Unprotect[qi`$opeHatSubscriptHermitian];
	AppendTo[qi`$opeHatSubscriptHermitian,HoldComplete[new]];
	Protect[qi`$opeHatSubscriptHermitian];
]; 


qi`setOpeHatSuper[new_]:=
Module[{},
	Unprotect[qi`$opeHatSuper];
	AppendTo[qi`$opeHatSuper,HoldComplete[new]];
	Protect[qi`$opeHatSuper];
]; 


qi`setOpeHatSubscriptSuper[new_]:=
Module[{},
	Unprotect[qi`$opeHatSubscriptSuper];
	AppendTo[qi`$opeHatSubscriptSuper,HoldComplete[new]];
	Protect[qi`$opeHatSubscriptSuper];
]; 


qi`setOpeHatSubPlusMinusSuper[new_]:=
Module[{},
	Unprotect[qi`$opeHatSubPlusMinusSuper];
	AppendTo[qi`$opeHatSubPlusMinusSuper,HoldComplete[new]];
	Protect[qi`$opeHatSubPlusMinusSuper];
]; 


(* Input *)

(* Auxiliary qmprg`commands, produced by one replacement and erased by 
another replacement *)
SetAttributes[qi`typesettingHoldComplete,HoldAllComplete]; 
SetAttributes[qi`typesettingTimes,HoldAllComplete]; 
(* Transform from numeric commutative functions, like Plus\[Rule]qe`plus *)
qi`setPreRules[qi`$numfunprerules];
qi`setPreRules[
	HoldPattern[TraditionalForm]->qe`traditionalform,
	HoldPattern[SuperStar]->qe`conjugate, (* IMPORTANT: Conjugate\[Rule]qe`conjugate is in qi`$numfunprerules*)
	HoldPattern[AngleBracket]->qe`expectation,
	HoldPattern[qm`Dirac`qmOverHat]->qe`opeHat,
	HoldPattern[qm`Dirac`qmKet]->qe`ket,
	HoldPattern[qm`Dirac`qmBra]->qe`bra,
	HoldPattern[qm`Dirac`qmBrkt]->qe`brkt,
	HoldPattern[qm`Dirac`qmQmutator]->qe`qmutator,
	HoldPattern[SuperDagger]->qe`hermitianConjugate,	
	HoldPattern[OverHat]->qe`opeHat,
	HoldPattern[Ket]->qe`ket,
	HoldPattern[Bra]->qe`bra,
	HoldPattern[BraKet]->qe`brkt,
	HoldPattern[Integrate]-> qe`integrate, 
	HoldPattern[Sum]-> qe`sum, 
	HoldPattern[Product]-> qe`product, 
	HoldPattern[NIntegrate]-> qe`nintegrate, 
	HoldPattern[NSum]-> qe`nsum, 
	HoldPattern[NProduct]-> qe`nproduct, 
	HoldPattern[Subscript]->qe`subscript,
	HoldPattern[Superscript]->qe`power, (* IMPORTANT Superscript\[Rule]qe`power *)
	HoldPattern[SubPlus]->qe`subPlus,
	HoldPattern[SubMinus]->qe`subMinus,
	HoldPattern[SuperPlus]->qe`superPlus,
	HoldPattern[SuperMinus]->qe`superMinus,

(* All the rules below this point must use only qe`Commands *)
	HoldPattern[qe`subscript[qe`qmutator[args___],subs_]]:>qe`qmutator[args,subs],
	HoldPattern[qe`subMinus[qe`qmutator[args___]]]:>qe`qmutator[args,1],
	HoldPattern[qe`subPlus[qe`qmutator[args___]]]:>qe`qmutator[args,-1],

	HoldPattern[qe`subscript[args___,\[ScriptS]]]:>qe`symmetric[args],

(* IMPORTANT qe`superscript from Power *)
	HoldPattern[qe`power[qe`ket[args___],expo_]]:>qe`superscript[qe`ket[args],expo],
	HoldPattern[qe`power[qe`bra[args___],expo_]]:>qe`superscript[qe`bra[args],expo],

(* qe`superscript to "symbolize" and avoid power, hermitian and others when needed *)
	HoldPattern[qe`hermitianConjugate[qe`opeHat[ope_]]]:>
		qe`superscript[qe`opeHat[ope],qi`dagger] /; 
			MemberQ[qi`$opeHatHermitian,HoldComplete[ope]],
	HoldPattern[qe`hermitianConjugate[qe`subscript[qe`opeHat[ope_],subs___]]]:>
		qe`superscript[qe`subscript[qe`opeHat[ope],subs],qi`dagger] /; 
			MemberQ[qi`$opeHatSubscriptHermitian,HoldComplete[ope]],
	HoldPattern[qe`power[qe`opeHat[ope_],expo_]]:>
		qe`superscript[qe`opeHat[ope],expo] /; 
			MemberQ[qi`$opeHatSuper,HoldComplete[ope]],
	HoldPattern[qe`power[qe`subscript[qe`opeHat[ope_],subs___],expo_]]:>
		qe`superscript[qe`subscript[qe`opeHat[ope],subs],expo] /; 
			MemberQ[qi`$opeHatSubscriptSuper,HoldComplete[ope]],
	HoldPattern[qe`power[qe`subPlus[qe`opeHat[ope_]],expo_]]:>
		qe`superscript[qe`subPlus[qe`opeHat[ope]],expo] /; 
			MemberQ[qi`$opeHatSubPlusMinusSuper,HoldComplete[ope]],
	HoldPattern[qe`power[qe`subMinus[qe`opeHat[ope_]],expo_]]:>
		qe`superscript[qe`subMinus[qe`opeHat[ope]],expo] /; 
			MemberQ[qi`$opeHatSubPlusMinusSuper,HoldComplete[ope]],

(* OverHat corrections *)
	HoldPattern[qe`opeHat[qe`subPlus[arg_]]]:>qe`subPlus[qe`opeHat[arg]],

	HoldPattern[qe`opeHat[qe`subMinus[arg_]]]:>qe`subMinus[qe`opeHat[arg]],

	HoldPattern[qe`opeHat[qe`subscript[ope_,subs___]]]:>qe`subscript[qe`opeHat[ope],subs],
	
(* It is important to use HoldPattern, HoldForm, Unevaluated, etc.
   in each of the Patterns, Rules, Conditions, Messages, etc. below this point *)

(* More OverHat corrections *)
	HoldPattern[qe`opeHat[ope_,args___]]:>
		(Message[qe`opeHat::qmunknown,HoldForm[qe`opeHat]];Abort[]) /;
			(If[And[qi`$verify,Head[Unevaluated[ope]]=!=Symbol],
				( Message[head::qmopename,HoldForm[ope]];Abort[] )];
			False),

	HoldPattern[qe`opeHat[ope_Symbol,args___]]:>
		(Message[qe`opeHat::qmunknown,HoldForm[qe`opeHat]];Abort[]) /;
			(If[And[qi`$verify,StringLength[SymbolName[Unevaluated[ope]]]>1],
				( Message[head::qmnamelen,HoldForm[ope]];Abort[] )];
			False),
			
(* When there is a power containing dagger symbol in the exponent,
		that dagger is interpreted as representing hermitian conjugate *)
	HoldPattern[exponent_Symbol]:>
		With[{my=
			Apply[qi`typesettingTimes,
				Map[ToExpression[#,StandardForm,qi`typesettingHoldComplete]&,
					DeleteCases[StringSplit[ToString[Unevaluated[exponent]],"\[Dagger]"->"\[Dagger]"],""]]]},
			my /; TrueQ[Length[my]>1] ],
	(* Auxiliary qmprg`commands, produced by one replacement and erased by
	another replacement *)
	qi`typesettingTimes->qe`times,
	HoldPattern[qi`typesettingHoldComplete[expr_]]:>Unevaluated[expr],
	HoldPattern[qe`power[base_,Global`\[Dagger]]]:> 
		qe`hermitianConjugate[base],
	HoldPattern[qe`power[Global`\[Dagger],expo_]]:> 
		qi`typesettingTimes[Global`\[Dagger],expo],
	HoldPattern[qe`power[base_,qe`times[Global`\[Dagger]]]]:> 
		qe`hermitianConjugate[base],
	HoldPattern[qe`power[base_,qe`times[Global`\[Dagger],oneargright_]]]:> 
		qe`power[qe`hermitianConjugate[base],oneargright],
	HoldPattern[qe`power[base_,qe`times[Global`\[Dagger],oneargright_,moreargright__]]]:> 
		qe`power[qe`hermitianConjugate[base],qi`typesettingTimes[oneargright,moreargright]],
	HoldPattern[qe`power[base_,qe`times[oneargleft_,Global`\[Dagger]]]]:>
		qe`hermitianConjugate[qe`power[base,oneargleft]],
	HoldPattern[qe`power[base_,qe`times[oneargleft_,Global`\[Dagger],oneargright_]]]:>
		qe`power[qe`hermitianConjugate[qe`power[base,oneargleft]],oneargright],
	HoldPattern[qe`power[base_,qe`times[oneargleft_,Global`\[Dagger],oneargright_,moreargright__]]]:>
		qe`power[qe`hermitianConjugate[qe`power[base,oneargleft]],
				qi`typesettingTimes[oneargright,moreargright]],
	HoldPattern[qe`power[base_,qe`times[oneargleft_,moreargleft__,Global`\[Dagger]]]]:>
		qe`hermitianConjugate[qe`power[base,qi`typesettingTimes[oneargleft,moreargleft]]],
	HoldPattern[qe`power[base_,qe`times[oneargleft_,moreargleft__,Global`\[Dagger],oneargright_]]]:>
		qe`power[qe`hermitianConjugate[qe`power[base,qi`typesettingTimes[oneargleft,moreargleft]]],oneargright],
	HoldPattern[qe`power[base_,qe`times[oneargleft_,moreargleft__,Global`\[Dagger],oneargright_,moreargright__]]]:>
		qe`power[qe`hermitianConjugate[qe`power[base,qi`typesettingTimes[oneargleft,moreargleft]]],
				qi`typesettingTimes[oneargright,moreargright]]
];


(* output *)
(* Transform to numeric commutative functions, like qe`plus\[Rule]Plus *)
qi`setPostRules[qi`$numfunposrules];
qi`setPostRules[
	HoldPattern[qe`superscript[base_,qi`dagger]]:>SuperDagger[base],
	HoldPattern[qe`symmetric[args___]]:>qe`subscript[args,\[ScriptS]],
	HoldPattern[qe`traditionalform]->TraditionalForm,
	HoldPattern[Conjugate]-> SuperStar, (* IMPORTANT: qe`conjugate is in qi`$numfunposrules*)
	HoldPattern[qe`superscript]->Power, (* IMPORTANT qe`superscript\[Rule]Power *)
	HoldPattern[qe`expectation]-> AngleBracket,
	HoldPattern[qe`hermitianConjugate]-> SuperDagger,
	HoldPattern[qe`opeHat]-> qm`Dirac`qmOverHat,
	HoldPattern[qe`ket]->qm`Dirac`qmKet,
	HoldPattern[qe`bra]->qm`Dirac`qmBra,
	HoldPattern[qe`brkt]->qm`Dirac`qmBrkt,
	HoldPattern[qe`qmutator]->qm`Dirac`qmQmutator,
    HoldPattern[qe`integrate]->Integrate, 
	HoldPattern[qe`sum]->Sum, 
	HoldPattern[qe`product]->Product,
    HoldPattern[qe`nintegrate]->NIntegrate, 
	HoldPattern[qe`nsum]->NSum, 
	HoldPattern[qe`nproduct]->NProduct,
	HoldPattern[qe`subscript]->Subscript,
	HoldPattern[qe`subPlus]->SubPlus,
	HoldPattern[qe`subMinus]->SubMinus,
	HoldPattern[qe`superPlus]->SuperPlus,
	HoldPattern[qe`superMinus]->SuperMinus
];


(*The HoldForm does not avoid evaluation, 
because qe`traditionalform does not have Hold Attributes,
however it is necesary so that TraditionalForm
renders ab-ba in the correct form instead of ab-ab
*)
qe`traditionalform[anyother_]:=TraditionalForm[HoldForm[anyother]];

Module[{unprotected},
	unprotected=Unprotect[qm`Dirac`qm];
	Format[HoldPattern[qm`Dirac`qm[expr_]],TraditionalForm]:=HoldForm[expr];
	Protect[Evaluate[unprotected]]
];


(* We change the output display of "subscript of subscript" to
	Subscript[(Subscript[a, b]), c] with parenthesis, so it is very clear the distintion between
	the subscipt of the slot and another subscript that belongs to the 
	label; therefore we have a consistent behavior of subscripted kets 
	vs kets of susbscripted arguments 
 *)
Module[{unprotected},

	unprotected=Unprotect[SubscriptBox];

	SubscriptBox[SubscriptBox[boxes1__],boxes2__]:=
		SubscriptBox[RowBox[{"(",SubscriptBox[boxes1],")"}],boxes2];
	SubscriptBox[RowBox[{box0_?(#=!="(" &),boxes1___}],boxes2__]:=
		SubscriptBox[RowBox[{"(",RowBox[{box0,boxes1}],")"}],boxes2];

	qi`setModifiedSymbols[SubscriptBox];

	Protect[Evaluate[unprotected]]
];


Module[{unprotected},

	unprotected=Unprotect[SuperscriptBox];

	SuperscriptBox[RowBox[{"(",SubscriptBox[mainbox_,subbox_],")"}],superbox_]:=
		SubsuperscriptBox[mainbox, subbox, superbox];

	qi`setModifiedSymbols[SuperscriptBox];
	Protect[Evaluate[unprotected]]
];


Unprotect[MakeBoxes];
qi`setModifiedSymbols[MakeBoxes];


SetAttributes[qi`brktTemplateBox,HoldAllComplete];
With[{alias="qmbraket"},
	qi`brktTemplateBox[box1_,box2_]:=
		TemplateBox[{box1,box2},"qm`Dirac`qmBrkt",
					InterpretationFunction:>
					(RowBox[{"qm`Dirac`qmBrkt","[",
							RowBox[{"{", #1, "}"}], ",",
							RowBox[{"{", #2, "}"}],
						"]"}]&),
					DisplayFunction:> 
					(TagBox[RowBox[{
							"\[LeftAngleBracket]",
							TagBox[#1,Sequence,BaseStyle->{ShowSyntaxStyles->True}],
							"\[VerticalLine]",
							TagBox[#2,Sequence,BaseStyle->{ShowSyntaxStyles->True}],
							"\[RightAngleBracket]"
								 }],qm`Dirac`qmBrkt,BaseStyle->{ShowSyntaxStyles->False}]&),
					Tooltip->"\[EscapeKey]"<>alias<>"\[EscapeKey]"];
	aliasbraket=alias;
];


MakeBoxes[qm`Dirac`qmBrkt[{args1__},{args2__}],StandardForm]:= 
	With[{rowbox1=RowBox[Riffle[Map[MakeBoxes,Unevaluated[{args1}]],","]],
		rowbox2=RowBox[Riffle[Map[MakeBoxes,Unevaluated[{args2}]],","]]},
		qi`brktTemplateBox[rowbox1,rowbox2]];

MakeBoxes[qm`Dirac`qmBrkt[{args1__},{args2__}],TraditionalForm]:= 
	With[{rowbox1=RowBox[Riffle[Map[MakeBoxes,Unevaluated[{args1}]],"\[InvisibleComma]"(*invisible comma*)]],
		rowbox2=RowBox[Riffle[Map[MakeBoxes,Unevaluated[{args2}]],"\[InvisibleComma]"(*invisible comma*)]]},
		qi`brktTemplateBox[rowbox1,rowbox2]];


SetAttributes[qi`braTemplateBox,HoldAllComplete];
With[{alias="qmbra"},
	qi`braTemplateBox[box1_]:=
		TemplateBox[{box1},"qm`Dirac`qmBra",
					InterpretationFunction->Automatic,
					DisplayFunction:>
					(TagBox[RowBox[{"\[LeftAngleBracket]",
							TagBox[#1,Sequence,BaseStyle->{ShowSyntaxStyles->True}],
							"\[VerticalLine]" }],"qm`Dirac`qmBra",BaseStyle->{ShowSyntaxStyles->False}]&),
					Tooltip->"\[EscapeKey]"<>alias<>"\[EscapeKey]"];
	aliasbra=alias;
];


MakeBoxes[qm`Dirac`qmBra[args___],StandardForm]:= 
	With[{rowbox=RowBox[Riffle[Map[MakeBoxes,Unevaluated[{args}]],","]]},
		qi`braTemplateBox[rowbox]];

MakeBoxes[qm`Dirac`qmBra[args___],TraditionalForm]:= 
	With[{rowbox=RowBox[Riffle[Map[MakeBoxes,Unevaluated[{args}]],"\[InvisibleComma]"(*invisible comma*)]]},
		qi`braTemplateBox[rowbox]];


SetAttributes[qi`ketTemplateBox,HoldAllComplete];
With[{alias="qmket"},
	qi`ketTemplateBox[box1_]:=
		TemplateBox[{box1},"qm`Dirac`qmKet",
					InterpretationFunction:>Automatic,
					DisplayFunction:>
					(TagBox[RowBox[{"\[VerticalLine]",
							TagBox[#1,Sequence,BaseStyle->{ShowSyntaxStyles->True}],
							"\[RightAngleBracket]" }],"qm`Dirac`qmKet",BaseStyle->{ShowSyntaxStyles->False}]&),
					Tooltip->"\[EscapeKey]"<>alias<>"\[EscapeKey]"];
	aliasket=alias;
];


MakeBoxes[qm`Dirac`qmKet[args___],StandardForm]:= 
	With[{rowbox=RowBox[Riffle[Map[MakeBoxes,Unevaluated[{args}]],","]]},
		qi`ketTemplateBox[rowbox]];

MakeBoxes[qm`Dirac`qmKet[args___],TraditionalForm]:= 
	With[{rowbox=RowBox[Riffle[Map[MakeBoxes,Unevaluated[{args}]],"\[InvisibleComma]"(*invisible comma*)]]},
		qi`ketTemplateBox[rowbox]];


With[{alias="qmqmut"},
	SetAttributes[qi`qmutatorTemplateBox,HoldAllComplete];
	qi`qmutatorTemplateBox[box1_,box2_]:=
		TemplateBox[{box1,box2},"qm`Dirac`qmQmutator",
					InterpretationFunction->Automatic,
					DisplayFunction:>
						(RowBox[{"\[LeftDoubleBracket]",#1,",",#2,"\[RightDoubleBracket]" }]&),
					Tooltip->"\[EscapeKey]"<>alias<>"\[EscapeKey]"];

	SetAttributes[qi`qmutatorTraditionalTemplateBox,HoldAllComplete];
	qi`qmutatorTraditionalTemplateBox[box1_,box2_]:=
		TemplateBox[{box1,box2},"qm`Dirac`qmQmutator",
					InterpretationFunction->Automatic,
					DisplayFunction:>
						(RowBox[{"[",#1,",",#2,"]" }]&),
					Tooltip->"\[EscapeKey]"<>alias<>"\[EscapeKey]"];
	aliasqmut=alias;
]


MakeBoxes[qm`Dirac`qmQmutator[a_,b_],StandardForm]:= 
	qi`qmutatorTemplateBox[ MakeBoxes[a,StandardForm],
							MakeBoxes[b,StandardForm]];
MakeBoxes[qm`Dirac`qmQmutator[a_,b_,q_],StandardForm]:= 
	SubscriptBox[
		qi`qmutatorTemplateBox[ MakeBoxes[a,StandardForm],
								MakeBoxes[b,StandardForm]],
		MakeBoxes[q,StandardForm]];
MakeBoxes[qm`Dirac`qmQmutator[a_,b_,1],StandardForm]:= 
	SubscriptBox[
		qi`qmutatorTemplateBox[ MakeBoxes[a,StandardForm],
								MakeBoxes[b,StandardForm]],
		"-"];
MakeBoxes[qm`Dirac`qmQmutator[a_,b_,-1],StandardForm]:= 
	SubscriptBox[
		qi`qmutatorTemplateBox[ MakeBoxes[a,StandardForm],
								MakeBoxes[b,StandardForm]],
		"+"];
MakeBoxes[qm`Dirac`qmQmutator[a_,b_],TraditionalForm]:= 
	qi`qmutatorTraditionalTemplateBox[  MakeBoxes[a,TraditionalForm],
										MakeBoxes[b,TraditionalForm]];
MakeBoxes[qm`Dirac`qmQmutator[a_,b_,q_],TraditionalForm]:= 
	SubscriptBox[
		qi`qmutatorTraditionalTemplateBox[  MakeBoxes[a,TraditionalForm],
											MakeBoxes[b,TraditionalForm]],
		MakeBoxes[q,TraditionalForm]];
MakeBoxes[qm`Dirac`qmQmutator[a_,b_,1],TraditionalForm]:= 
	SubscriptBox[
		qi`qmutatorTraditionalTemplateBox[  MakeBoxes[a,TraditionalForm],
											MakeBoxes[b,TraditionalForm]],
		"-"];
MakeBoxes[qm`Dirac`qmQmutator[a_,b_,-1],TraditionalForm]:= 
	SubscriptBox[
		qi`qmutatorTraditionalTemplateBox[  MakeBoxes[a,TraditionalForm],
											MakeBoxes[b,TraditionalForm]],
		"+"];


SetAttributes[qi`overHatTemplateBox,HoldAllComplete];
With[{alias="qmoper"},
	qi`overHatTemplateBox[box1_]:=
		TemplateBox[{box1},"qm`Dirac`qmOverHat",
					InterpretationFunction:>Automatic,
					DisplayFunction:>(OverscriptBox[#1,"^"]&),
					Tooltip->"\[EscapeKey]"<>alias<>"\[EscapeKey]"];
	aliasoverHat=alias;
];


MakeBoxes[qm`Dirac`qmOverHat[a_],form_]:= 
	qi`overHatTemplateBox[MakeBoxes[a,form]];


MakeBoxes[SuperDagger[Subscript[base_,onesubs_]],form_]:=
	SubsuperscriptBox[MakeBoxes[base,form],MakeBoxes[onesubs,form],InterpretationBox["\[Dagger]",Global`\[Dagger],Editable->False]];


MakeBoxes[SuperDagger[Subscript[base_,subs1_,moresubs__]],form_]:=
With[{rowbox=RowBox[Riffle[Map[MakeBoxes,Unevaluated[{subs1,moresubs}]],","]]},
	SubsuperscriptBox[MakeBoxes[base,form],rowbox,InterpretationBox["\[Dagger]",Global`\[Dagger],Editable->False]]];


(* Finishing this package *)


qi`setCommands[
	qi`setOpeHatHermitian,
	qi`setOpeHatSubscriptHermitian,
	qi`setOpeHatSuper,
	qi`setOpeHatSubscriptSuper,
	qi`setOpeHatSubPlusMinusSuper,
	qi`braTemplateBox,
	qi`brktTemplateBox,
	qi`ketTemplateBox,
	qi`overHatTemplateBox,
	qi`qmutatorTemplateBox,
	qi`qmutatorTraditionalTemplateBox
];

qi`setUnlockedCommands[
	qe`traditionalform,
	qe`opeHat,  
	qe`hermitianConjugate, qe`expectation,
	qe`subscript,
	qe`superscript,
	qe`subPlus,
	qe`subMinus,
	qe`superPlus,
	qe`superMinus,
	qe`qmutator,
	qe`symmetric, 
	qe`ket, qe`bra, qe`brkt
];

qi`setUnlockedCommands[
	qi`$numfunprerules[[All,2]]
];


(* Error Messages*)
General::qmopename="Symbol expected instead of `1` as operator label";
General::qmnamelen="One letter expected instead of `1` as operator label";


With[{box=ToBoxes[Unevaluated[qm`Dirac`qmKet[\[SelectionPlaceholder]]]]},
qi`setAlias[aliasket,
				box,
				Row[{DisplayForm[box],
					" ket template "
					}]]];
With[{box=ToBoxes[Unevaluated[qm`Dirac`qmBra[\[SelectionPlaceholder]]]]},
qi`setAlias[aliasbra,
				box,
				Row[{DisplayForm[box],
					" bra template "
					}]]];
With[{box=ToBoxes[Unevaluated[qm`Dirac`qmBrkt[{\[SelectionPlaceholder]},{\[Placeholder]}]]]},
qi`setAlias[aliasbraket,
				box,
				Row[{DisplayForm[box],
					" braket template "
					}]]];
With[{box=ToBoxes[Unevaluated[Sum[\[SelectionPlaceholder],{\[Placeholder],\[Placeholder],\[Placeholder]}]]]},				
qi`setAlias["qmsum",
				box,
				Row[{DisplayForm[box],
					" sum template "
					}]]];
With[{box=ToBoxes[Unevaluated[Integrate[\[SelectionPlaceholder],{\[Placeholder],\[Placeholder],\[Placeholder]}]]]},				
qi`setAlias["qmint",
				box,
				Row[{DisplayForm[box],
					" definite integral template "
					}]]];
With[{box=ToBoxes[Unevaluated[Product[\[SelectionPlaceholder],{\[Placeholder],\[Placeholder],\[Placeholder]}]]]}, 
qi`setAlias["qmprd",
				box,
				Row[{DisplayForm[box],
					" product template "
					}]]];
With[{box=ToBoxes[Unevaluated[Power[\[SelectionPlaceholder],\[Placeholder]]]]}, 
qi`setAlias["qmpowe",
				box,
				Row[{DisplayForm[box],
					" power template "
					}]]];
With[{box=ToBoxes[Unevaluated[SuperStar[(\[SelectionPlaceholder])]]]}, 
qi`setAlias["qmconj",
				box,
				Row[{DisplayForm[box],
					" complex conjugate template "
					}]]];
With[{box=ToBoxes[Unevaluated[SuperDagger[(\[SelectionPlaceholder])]]]}, 
qi`setAlias[aliassuperDagger,
				box,
				Row[{DisplayForm[box],
					" hermitian conjugate template "
					}]]];
With[{box=ToBoxes[Unevaluated[Subscript[\[SelectionPlaceholder],\[Placeholder]]]]}, 
qi`setAlias["qmsubs",
				box,
				Row[{DisplayForm[box],
					" subscript template "
					}]]];
With[{box=ToBoxes[Unevaluated[qm`Dirac`qmOverHat[\[SelectionPlaceholder]]]]}, 
qi`setAlias[aliasoverHat,
				box,
				Row[{DisplayForm[box],
					"  operator template "
					}]]];
With[{box=ToBoxes[Unevaluated[Subscript[qm`Dirac`qmOverHat[\[SelectionPlaceholder]], \[Placeholder]]]]}, 
qi`setAlias["qmopes",
				box,
				Row[{DisplayForm[box],
					" subscripted operator template "
					}]]];
With[{box=ToBoxes[Unevaluated[Power[qm`Dirac`qmOverHat[\[SelectionPlaceholder]],\[Placeholder]]]]}, 
qi`setAlias["qmopep",
				box,
				Row[{DisplayForm[box],
					" power of operator template "
					}]]];
With[{box=ToBoxes[Unevaluated[SuperDagger[qm`Dirac`qmOverHat[\[SelectionPlaceholder]]]]]}, 
qi`setAlias["qmopeh",
				box,
				Row[{DisplayForm[box],
					" hermitian of operator template "
					}]]];
With[{box=ToBoxes[Unevaluated[Power[Subscript[qm`Dirac`qmOverHat[\[SelectionPlaceholder]], \[Placeholder]],\[Placeholder]]]]}, 
qi`setAlias["qmopps",
				box,
				Row[{DisplayForm[box],
					" power of subscripted operator "
					}]]];
With[{box=ToBoxes[Unevaluated[Power[Subscript[qm`Dirac`qmOverHat[\[SelectionPlaceholder]], \[Placeholder]],\[Placeholder]]]]}, 
qi`setAlias["qmopsp",
				box,
				Row[{DisplayForm[box],
					" power of subscripted operator "
					}]]];
With[{box=ToBoxes[Unevaluated[SuperDagger[Subscript[qm`Dirac`qmOverHat[\[SelectionPlaceholder]], \[Placeholder]]]]]}, 
qi`setAlias["qmophs",
				box,
				Row[{DisplayForm[box],
					" hermitian of subscripted operator "
					}]]];
With[{box=ToBoxes[Unevaluated[SuperDagger[Subscript[qm`Dirac`qmOverHat[\[SelectionPlaceholder]], \[Placeholder]]]]]}, 
qi`setAlias["qmopsh",
				box,
				Row[{DisplayForm[box],
					" hermitian of subscripted operator "
					}]]];
With[{box=ToBoxes[Unevaluated[qm`Dirac`qmQmutator[\[SelectionPlaceholder],\[Placeholder],1]]]}, 
qi`setAlias["qmcomm",
				box,
				Row[{DisplayForm[box],
					" commutator template "
					}]]];
With[{box=ToBoxes[Unevaluated[qm`Dirac`qmQmutator[\[SelectionPlaceholder],\[Placeholder],-1]]]}, 
qi`setAlias["qmanti",
				box,
				Row[{DisplayForm[box],
					" anticommutator template "
					}]]];
With[{box=ToBoxes[Unevaluated[qm`Dirac`qmQmutator[\[SelectionPlaceholder],\[Placeholder],\[Placeholder]]]]}, 
qi`setAlias[aliasqmut,
				box,
				Row[{DisplayForm[box],
					" q-mutator template "
					}]]];					
With[{box=ToBoxes[Unevaluated[AngleBracket[\[SelectionPlaceholder]]]]}, 
qi`setAlias["qmexpe",
				box,
				Row[{DisplayForm[box],
					" expectation template "
					}]]];


qi`$initialPaletteTabIndex="Typesetting";
qi`setPaletteTab["Typesetting",12,
			{
				PasteButton[Defer[qm`Dirac`qm[\!\(\*
TagBox[
FrameBox["\<\"expression\"\>"],
"SelectionPlaceholder"]\)]]],
				SpanFromLeft,SpanFromLeft,SpanFromLeft,SpanFromLeft,SpanFromLeft,

				PasteButton[Defer[\!\(\*
TagBox[
RowBox[{"Integrate", "[", 
RowBox[{"\[SelectionPlaceholder]", ",", 
RowBox[{"{", 
RowBox[{"\[Placeholder]", ",", "\[Placeholder]", ",", "\[Placeholder]"}], "}"}]}], "]"}],
DisplayForm]\)]],
				SpanFromLeft, 
				PasteButton[Defer[\!\(\*
TagBox[
RowBox[{"Sum", "[", 
RowBox[{"\[SelectionPlaceholder]", ",", 
RowBox[{"{", 
RowBox[{"\[Placeholder]", ",", "\[Placeholder]", ",", "\[Placeholder]"}], "}"}]}], "]"}],
DisplayForm]\)]],
				SpanFromLeft,
				PasteButton[Defer[\!\(\*
TagBox[
RowBox[{"Product", "[", 
RowBox[{"\[SelectionPlaceholder]", ",", 
RowBox[{"{", 
RowBox[{"\[Placeholder]", ",", "\[Placeholder]", ",", "\[Placeholder]"}], "}"}]}], "]"}],
DisplayForm]\)]],
				SpanFromLeft,
			
				PasteButton[Tooltip["\[LeftAngleBracket]\[SelectionPlaceholder]\[VerticalSeparator]",aliasbra],Defer[\!\(\*
TagBox[
RowBox[{"qm`Dirac`qmBra", "[", "\[SelectionPlaceholder]", "]"}],
DisplayForm]\)]], SpanFromLeft,
				PasteButton[Tooltip["\[LeftAngleBracket]\[SelectionPlaceholder]\[VerticalSeparator]\[Placeholder]\[RightAngleBracket]",aliasbraket],Defer[\!\(\*
TagBox[
RowBox[{"qm`Dirac`qmBrkt", "[", 
RowBox[{
RowBox[{"{", "\[SelectionPlaceholder]", "}"}], ",", 
RowBox[{"{", "\[Placeholder]", "}"}]}], "]"}],
DisplayForm]\)]], SpanFromLeft,
				PasteButton[Tooltip["\[VerticalSeparator]\[SelectionPlaceholder]\[RightAngleBracket]",aliasket],Defer[\!\(\*
TagBox[
RowBox[{"qm`Dirac`qmKet", "[", "\[SelectionPlaceholder]", "]"}],
DisplayForm]\)]], SpanFromLeft,
				
				PasteButton[Defer[qm`Dirac`qmQmutator[\[SelectionPlaceholder],\[Placeholder],1]]],
				SpanFromLeft, 
				PasteButton[Defer[qm`Dirac`qmQmutator[\[SelectionPlaceholder],\[Placeholder],-1]]],
				SpanFromLeft, 
				PasteButton[Defer[qm`Dirac`qmQmutator[\[SelectionPlaceholder],\[Placeholder],\[Placeholder]]]],
				SpanFromLeft,

				PasteButton[Defer[qm`Dirac`qmOverHat[\[SelectionPlaceholder]]]],
				PasteButton[Defer[Power[qm`Dirac`qmOverHat[\[SelectionPlaceholder]],\[Placeholder]]]],
				PasteButton[Defer[SuperDagger[qm`Dirac`qmOverHat[\[SelectionPlaceholder]]]]],
				PasteButton[Defer[Subscript[qm`Dirac`qmOverHat[\[SelectionPlaceholder]], \[Placeholder]]]],
				PasteButton[Defer[Power[Subscript[qm`Dirac`qmOverHat[\[SelectionPlaceholder]], \[Placeholder]],\[Placeholder]]]],
				PasteButton[Defer[SuperDagger[Subscript[qm`Dirac`qmOverHat[\[SelectionPlaceholder]], \[Placeholder]]]]],

				PasteButton[Defer[AngleBracket[\[SelectionPlaceholder]]]],
				PasteButton[Defer[Power[\[SelectionPlaceholder],\[Placeholder]]]],
				PasteButton[Defer[SuperDagger[(\[SelectionPlaceholder])]]],
				PasteButton[Defer[Subscript[\[SelectionPlaceholder],\[Placeholder]]]],
				PasteButton[Defer[Power[Subscript[\[SelectionPlaceholder], \[Placeholder]],\[Placeholder]]]],
				PasteButton[Defer[SuperStar[(\[SelectionPlaceholder])]]],
			
				PasteButton[Defer[Pi]],
				PasteButton[Defer[I]],
				PasteButton[Defer[E]],
				PasteButton[Defer[Global`\[HBar]]],
				PasteButton[Defer[\!\(\*
TagBox["Global`\[ScriptCapitalH]",
DisplayForm]\)]],				
				PasteButton[Defer[Global`\[ScriptCapitalC]]],
				PasteButton[Defer[Global`\[ScriptCapitalD]]],
				PasteButton[Defer[\!\(\*
TagBox["Global`\[ScriptCapitalE]",
DisplayForm]\)]],
				PasteButton[Defer[\!\(\*
TagBox["Global`\[ScriptCapitalI]",
DisplayForm]\)]],
				PasteButton[Defer[\!\(\*
TagBox["Global`\[ScriptCapitalK]",
DisplayForm]\)]],
				PasteButton[Defer[\!\(\*
TagBox["Global`\[ScriptCapitalN]",
DisplayForm]\)]],
				PasteButton[Defer[\!\(\*
TagBox["Global`\[ScriptCapitalO]",
DisplayForm]\)]],				
				PasteButton[Defer[Global`\[Psi]]],
				PasteButton[Defer[Global`\[Phi]]],
				PasteButton[Defer[Global`\[CurlyPhi]]],
				PasteButton[Defer[\!\(\*
TagBox["Global`\[Sigma]",
DisplayForm]\)]],
				PasteButton[Defer[Global`\[Alpha]]],
				PasteButton[Defer[Global`\[Beta]]],
				PasteButton[Defer[Global`\[Gamma]]],
				PasteButton[Defer[Global`\[Delta]]],
				PasteButton[Defer[Global`\[Epsilon]]],
				PasteButton[Defer[\!\(\*
TagBox["Global`\[Xi]",
DisplayForm]\)]],
				PasteButton[Defer[\!\(\*
TagBox["Global`\[Eta]",
DisplayForm]\)]],
				PasteButton[Defer[\!\(\*
TagBox["Global`\[Theta]",
DisplayForm]\)]]
			}
];


End[];


qi`message=qi`hello[];


EndPackage[];

Column[{qi`message, qi`sessionAliases[]}]
